@@ -18,14 +18,13 @@ import contextlib
 
 import six
 
+from neutron_lib.api.definitions import port_security as psec
 from neutron_lib import constants as n_const
+from neutron_lib.utils import runtime
 
-import neutron.common.constants as const
-from neutron.common import utils
-from neutron.extensions import portsecurity as psec
 
-INGRESS_DIRECTION = const.INGRESS_DIRECTION
-EGRESS_DIRECTION = const.EGRESS_DIRECTION
+INGRESS_DIRECTION = n_const.INGRESS_DIRECTION
+EGRESS_DIRECTION = n_const.EGRESS_DIRECTION
 
 DIRECTION_IP_PREFIX = {INGRESS_DIRECTION: 'source_ip_prefix',
                        EGRESS_DIRECTION: 'dest_ip_prefix'}
@@ -33,10 +32,15 @@ DIRECTION_IP_PREFIX = {INGRESS_DIRECTION: 'source_ip_prefix',
 # List of ICMPv6 types that should be permitted (ingress) by default. This list
 # depends on iptables conntrack behavior of recognizing ICMP errors (types 1-4)
 # as related traffic.
-ICMPV6_ALLOWED_TYPES = [n_const.ICMPV6_TYPE_MLD_QUERY,
-                        n_const.ICMPV6_TYPE_RA,
-                        n_const.ICMPV6_TYPE_NS,
-                        n_const.ICMPV6_TYPE_NA]
+ICMPV6_ALLOWED_INGRESS_TYPES = (n_const.ICMPV6_TYPE_MLD_QUERY,
+                                n_const.ICMPV6_TYPE_NS,
+                                n_const.ICMPV6_TYPE_NA)
+
+# List of ICMPv6 types that should be permitted (egress) by default.
+ICMPV6_ALLOWED_EGRESS_TYPES = (n_const.ICMPV6_TYPE_MLD_QUERY,
+                               n_const.ICMPV6_TYPE_RS,
+                               n_const.ICMPV6_TYPE_NS,
+                               n_const.ICMPV6_TYPE_NA)
 
 
 def port_sec_enabled(port):
@@ -44,7 +48,7 @@ def port_sec_enabled(port):
 
 
 def load_firewall_driver_class(driver):
-    return utils.load_class_by_alias_or_classname(
+    return runtime.load_class_by_alias_or_classname(
         'neutron.agent.firewall_drivers', driver)
 
 
@@ -155,6 +159,13 @@ class FirewallDriver(object):
         """
         raise NotImplementedError()
 
+    def process_trusted_ports(self, port_ids):
+        """Process ports that are trusted and shouldn't be filtered."""
+        pass
+
+    def remove_trusted_ports(self, port_ids):
+        pass
+
 
 class NoopFirewallDriver(FirewallDriver):
     """Noop Firewall Driver.
