@@ -16,43 +16,21 @@
 import os
 
 import eventlet
-from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import fileutils
 import webob
 
-from neutron._i18n import _, _LI
-from neutron.agent.linux import keepalived
 from neutron.agent.linux import utils as agent_utils
-from neutron.common import utils as common_utils
+from neutron.common import constants
 from neutron.notifiers import batch_notifier
 
 LOG = logging.getLogger(__name__)
 
 KEEPALIVED_STATE_CHANGE_SERVER_BACKLOG = 4096
 
-OPTS = [
-    cfg.StrOpt('ha_confs_path',
-               default='$state_path/ha_confs',
-               help=_('Location to store keepalived/conntrackd '
-                      'config files')),
-    cfg.StrOpt('ha_vrrp_auth_type',
-               default='PASS',
-               choices=keepalived.VALID_AUTH_TYPES,
-               help=_('VRRP authentication type')),
-    cfg.StrOpt('ha_vrrp_auth_password',
-               help=_('VRRP authentication password'),
-               secret=True),
-    cfg.IntOpt('ha_vrrp_advert_int',
-               default=2,
-               help=_('The advertisement interval in seconds')),
-    cfg.IntOpt('ha_keepalived_state_change_server_threads',
-               default=(1 + common_utils.cpu_count()) // 2,
-               min=1,
-               help=_('Number of concurrent threads for '
-                      'keepalived server connection requests.'
-                      'More threads create a higher CPU load '
-                      'on the agent node.')),
-]
+TRANSLATION_MAP = {'master': constants.HA_ROUTER_STATE_ACTIVE,
+                   'backup': constants.HA_ROUTER_STATE_STANDBY,
+                   'fault': constants.HA_ROUTER_STATE_STANDBY}
 
 
 class KeepalivedStateChangeHandler(object):
@@ -99,56 +77,77 @@ class AgentMixin(object):
     def __init__(self, host):
         self._init_ha_conf_path()
         super(AgentMixin, self).__init__(host)
+        # BatchNotifier queue is needed to ensure that the HA router
+        # state change sequence is under the proper order.
         self.state_change_notifier = batch_notifier.BatchNotifier(
             self._calculate_batch_duration(), self.notify_server)
         eventlet.spawn(self._start_keepalived_notifications_server)
 
+    def _get_router_info(self, router_id):
+        try:
+            return self.router_info[router_id]
+        except KeyError:
+            LOG.info('Router %s is not managed by this agent. It was '
+                     'possibly deleted concurrently.', router_id)
+
+    def check_ha_state_for_router(self, router_id, current_state):
+        ri = self._get_router_info(router_id)
+        if ri and current_state != TRANSLATION_MAP[ri.ha_state]:
+            LOG.debug("Updating server with state %(state)s for router "
+                      "%(router_id)s", {'router_id': router_id,
+                                        'state': ri.ha_state})
+            self.state_change_notifier.queue_event((router_id, ri.ha_state))
+
     def _start_keepalived_notifications_server(self):
         state_change_server = (
             L3AgentKeepalivedStateChangeServer(self, self.conf))
         state_change_server.run()
 
     def _calculate_batch_duration(self):
-        # Slave becomes the master after not hearing from it 3 times
-        detection_time = self.conf.ha_vrrp_advert_int * 3
-
-        # Keepalived takes a couple of seconds to configure the VIPs
-        configuration_time = 2
-
-        # Give it enough slack to batch all events due to the same failure
-        return (detection_time + configuration_time) * 2
+        # Set the BatchNotifier interval to ha_vrrp_advert_int,
+        # default 2 seconds.
+        return self.conf.ha_vrrp_advert_int
 
     def enqueue_state_change(self, router_id, state):
-        LOG.info(_LI('Router %(router_id)s transitioned to %(state)s'),
-                 {'router_id': router_id,
-                  'state': state})
+        state_change_data = {"router_id": router_id, "state": state}
+        LOG.info('Router %(router_id)s transitioned to %(state)s',
+                 state_change_data)
 
-        try:
-            ri = self.router_info[router_id]
-        except KeyError:
-            LOG.info(_LI('Router %s is not managed by this agent. It was '
-                         'possibly deleted concurrently.'), router_id)
+        ri = self._get_router_info(router_id)
+        if ri is None:
             return
 
-        self._configure_ipv6_ra_on_ext_gw_port_if_necessary(ri, state)
+        # TODO(dalvarez): Fix bug 1677279 by moving the IPv6 parameters
+        # configuration to keepalived-state-change in order to remove the
+        # dependency that currently exists on l3-agent running for the IPv6
+        # failover.
+        self._configure_ipv6_params_on_ext_gw_port_if_necessary(ri, state)
         if self.conf.enable_metadata_proxy:
             self._update_metadata_proxy(ri, router_id, state)
         self._update_radvd_daemon(ri, state)
+        self.pd.process_ha_state(router_id, state == 'master')
         self.state_change_notifier.queue_event((router_id, state))
+        self.l3_ext_manager.ha_state_change(self.context, state_change_data)
 
-    def _configure_ipv6_ra_on_ext_gw_port_if_necessary(self, ri, state):
+    def _configure_ipv6_params_on_ext_gw_port_if_necessary(self, ri, state):
         # If ipv6 is enabled on the platform, ipv6_gateway config flag is
         # not set and external_network associated to the router does not
         # include any IPv6 subnet, enable the gateway interface to accept
-        # Router Advts from upstream router for default route.
+        # Router Advts from upstream router for default route on master
+        # instances as well as ipv6 forwarding. Otherwise, disable them.
         ex_gw_port_id = ri.ex_gw_port and ri.ex_gw_port['id']
-        if state == 'master' and ex_gw_port_id:
-            interface_name = ri.get_external_device_name(ex_gw_port_id)
-            if ri.router.get('distributed', False):
-                namespace = ri.ha_namespace
-            else:
-                namespace = ri.ns_name
-            ri._enable_ra_on_gw(ri.ex_gw_port, namespace, interface_name)
+        if not ex_gw_port_id:
+            return
+
+        interface_name = ri.get_external_device_name(ex_gw_port_id)
+        if ri.router.get('distributed', False):
+            namespace = ri.ha_namespace
+        else:
+            namespace = ri.ns_name
+
+        enable = state == 'master'
+        ri._configure_ipv6_params_on_gw(ri.ex_gw_port, namespace,
+                                        interface_name, enable)
 
     def _update_metadata_proxy(self, ri, router_id, state):
         if state == 'master':
@@ -159,7 +158,7 @@ class AgentMixin(object):
         else:
             LOG.debug('Closing metadata proxy for router %s', router_id)
             self.metadata_driver.destroy_monitored_metadata_proxy(
-                self.process_monitor, ri.router_id, self.conf)
+                self.process_monitor, ri.router_id, self.conf, ri.ns_name)
 
     def _update_radvd_daemon(self, ri, state):
         # Radvd has to be spawned only on the Master HA Router. If there are
@@ -170,10 +169,7 @@ class AgentMixin(object):
             ri.disable_radvd()
 
     def notify_server(self, batched_events):
-        translation_map = {'master': 'active',
-                           'backup': 'standby',
-                           'fault': 'standby'}
-        translated_states = dict((router_id, translation_map[state]) for
+        translated_states = dict((router_id, TRANSLATION_MAP[state]) for
                                  router_id, state in batched_events)
         LOG.debug('Updating server with HA routers states %s',
                   translated_states)
@@ -182,4 +178,4 @@ class AgentMixin(object):
 
     def _init_ha_conf_path(self):
         ha_full_path = os.path.dirname("/%s/" % self.conf.ha_confs_path)
-        common_utils.ensure_dir(ha_full_path)
+        fileutils.ensure_tree(ha_full_path, mode=0o755)
