@@ -14,23 +14,19 @@
 #    under the License.
 
 import collections
-import re
 
 import netaddr
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
-import six
 
-from neutron._i18n import _LI
 from neutron.agent import firewall
 from neutron.agent.linux import ip_conntrack
 from neutron.agent.linux import ipset_manager
 from neutron.agent.linux import iptables_comments as ic
 from neutron.agent.linux import iptables_manager
-from neutron.agent.linux import utils
-from neutron.common import exceptions as n_exc
+from neutron.common import constants as n_const
 from neutron.common import ipv6_utils
 from neutron.common import utils as c_utils
 
@@ -43,15 +39,11 @@ CHAIN_NAME_PREFIX = {firewall.INGRESS_DIRECTION: 'i',
                      SPOOF_FILTER: 's'}
 IPSET_DIRECTION = {firewall.INGRESS_DIRECTION: 'src',
                    firewall.EGRESS_DIRECTION: 'dst'}
-# length of all device prefixes (e.g. qvo, tap, qvb)
-LINUX_DEV_PREFIX_LEN = 3
-LINUX_DEV_LEN = 14
-MAX_CONNTRACK_ZONES = 65535
 comment_rule = iptables_manager.comment_rule
 
 
 def get_hybrid_port_name(port_name):
-    return (constants.TAP_DEVICE_PREFIX + port_name)[:LINUX_DEV_LEN]
+    return (constants.TAP_DEVICE_PREFIX + port_name)[:n_const.LINUX_DEV_LEN]
 
 
 class mac_iptables(netaddr.mac_eui48):
@@ -63,20 +55,22 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
     """Driver which enforces security groups through iptables rules."""
     IPTABLES_DIRECTION = {firewall.INGRESS_DIRECTION: 'physdev-out',
                           firewall.EGRESS_DIRECTION: 'physdev-in'}
+    CONNTRACK_ZONE_PER_PORT = False
 
     def __init__(self, namespace=None):
-        self.iptables = iptables_manager.IptablesManager(
-            use_ipv6=ipv6_utils.is_enabled(),
+        self.iptables = iptables_manager.IptablesManager(state_less=True,
+            use_ipv6=ipv6_utils.is_enabled_and_bind_by_default(),
             namespace=namespace)
         # TODO(majopela, shihanzhang): refactor out ipset to a separate
         # driver composed over this one
         self.ipset = ipset_manager.IpsetManager(namespace=namespace)
-        self.ipconntrack = ip_conntrack.IpConntrackManager(
-            self.get_device_zone, namespace=namespace)
-        self._populate_initial_zone_map()
         # list of port which has security group
         self.filtered_ports = {}
         self.unfiltered_ports = {}
+        self.ipconntrack = ip_conntrack.get_conntrack(
+            self.iptables.get_rules_for_table, self.filtered_ports,
+            self.unfiltered_ports, namespace=namespace,
+            zone_per_port=self.CONNTRACK_ZONE_PER_PORT)
         self._add_fallback_chain_v4v6()
         self._defer_apply = False
         self._pre_defer_filtered_ports = None
@@ -89,36 +83,10 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             lambda: collections.defaultdict(list))
         self.pre_sg_members = None
         self.enable_ipset = cfg.CONF.SECURITYGROUP.enable_ipset
-        self._enabled_netfilter_for_bridges = False
         self.updated_rule_sg_ids = set()
         self.updated_sg_members = set()
         self.devices_with_updated_sg_members = collections.defaultdict(list)
 
-    def _enable_netfilter_for_bridges(self):
-        # we only need to set these values once, but it has to be when
-        # we create a bridge; before that the bridge module might not
-        # be loaded and the proc values aren't there.
-        if self._enabled_netfilter_for_bridges:
-            return
-        else:
-            self._enabled_netfilter_for_bridges = True
-
-        # These proc values ensure that netfilter is enabled on
-        # bridges; essential for enforcing security groups rules with
-        # OVS Hybrid.  Distributions can differ on whether this is
-        # enabled by default or not (Ubuntu - yes, Redhat - no, for
-        # example).
-        LOG.debug("Enabling netfilter for bridges")
-        utils.execute(['sysctl', '-w',
-                       'net.bridge.bridge-nf-call-arptables=1'],
-                      run_as_root=True)
-        utils.execute(['sysctl', '-w',
-                       'net.bridge.bridge-nf-call-ip6tables=1'],
-                      run_as_root=True)
-        utils.execute(['sysctl', '-w',
-                       'net.bridge.bridge-nf-call-iptables=1'],
-                      run_as_root=True)
-
     @property
     def ports(self):
         return dict(self.filtered_ports, **self.unfiltered_ports)
@@ -148,8 +116,18 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         LOG.debug("Update members of security group (%s)", sg_id)
         self.sg_members[sg_id] = collections.defaultdict(list, sg_members)
         if self.enable_ipset:
-            for ip_version, current_ips in sg_members.items():
-                self.ipset.set_members(sg_id, ip_version, current_ips)
+            self._update_ipset_members(sg_id, sg_members)
+
+    def _update_ipset_members(self, sg_id, sg_members):
+        devices = self.devices_with_updated_sg_members.pop(sg_id, None)
+        for ip_version, current_ips in sg_members.items():
+            add_ips, del_ips = self.ipset.set_members(
+                sg_id, ip_version, current_ips)
+            if devices and del_ips:
+                # remove prefix from del_ips
+                ips = [str(netaddr.IPNetwork(del_ip).ip) for del_ip in del_ips]
+                self.ipconntrack.delete_conntrack_state_by_remote_ips(
+                    devices, ip_version, ips)
 
     def _set_ports(self, port):
         if not firewall.port_sec_enabled(port):
@@ -163,11 +141,17 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         self.unfiltered_ports.pop(port['device'], None)
         self.filtered_ports.pop(port['device'], None)
 
+    def _remove_conntrack_entries_from_port_deleted(self, port):
+        device_info = self.filtered_ports.get(port['device'])
+        if not device_info:
+            return
+        for ethertype in [constants.IPv4, constants.IPv6]:
+            self.ipconntrack.delete_conntrack_state_by_remote_ips(
+                [device_info], ethertype, set())
+
     def prepare_port_filter(self, port):
         LOG.debug("Preparing device (%s) filter", port['device'])
-        self._remove_chains()
         self._set_ports(port)
-        self._enable_netfilter_for_bridges()
         # each security group has it own chains
         self._setup_chains()
         return self.iptables.apply()
@@ -175,8 +159,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
     def update_port_filter(self, port):
         LOG.debug("Updating device (%s) filter", port['device'])
         if port['device'] not in self.ports:
-            LOG.info(_LI('Attempted to update port filter which is not '
-                         'filtered %s'), port['device'])
+            LOG.info('Attempted to update port filter which is not '
+                     'filtered %s', port['device'])
             return
         self._remove_chains()
         self._set_ports(port)
@@ -186,10 +170,11 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
     def remove_port_filter(self, port):
         LOG.debug("Removing device (%s) filter", port['device'])
         if port['device'] not in self.ports:
-            LOG.info(_LI('Attempted to remove port filter which is not '
-                         'filtered %r'), port)
+            LOG.info('Attempted to remove port filter which is not '
+                     'filtered %r', port)
             return
         self._remove_chains()
+        self._remove_conntrack_entries_from_port_deleted(port)
         self._unset_ports(port)
         self._setup_chains()
         return self.iptables.apply()
@@ -219,6 +204,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         # agent restarts and don't cause unnecessary rule differences
         for pname in sorted(ports):
             port = ports[pname]
+            self._add_conntrack_jump(port)
             self._setup_chain(port, firewall.INGRESS_DIRECTION)
             self._setup_chain(port, firewall.EGRESS_DIRECTION)
         self.iptables.ipv4['filter'].add_rule(SG_CHAIN, '-j ACCEPT')
@@ -239,6 +225,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._remove_chain(port, firewall.INGRESS_DIRECTION)
             self._remove_chain(port, firewall.EGRESS_DIRECTION)
             self._remove_chain(port, SPOOF_FILTER)
+            self._remove_conntrack_jump(port)
         for port in unfiltered_ports.values():
             self._remove_rule_port_sec(port, firewall.INGRESS_DIRECTION)
             self._remove_rule_port_sec(port, firewall.EGRESS_DIRECTION)
@@ -260,18 +247,10 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         self.iptables.ipv6['filter'].add_rule('sg-fallback', '-j DROP',
                                               comment=ic.UNMATCH_DROP)
 
-    def _add_raw_chain(self, chain_name):
-        self.iptables.ipv4['raw'].add_chain(chain_name)
-        self.iptables.ipv6['raw'].add_chain(chain_name)
-
     def _add_chain_by_name_v4v6(self, chain_name):
         self.iptables.ipv4['filter'].add_chain(chain_name)
         self.iptables.ipv6['filter'].add_chain(chain_name)
 
-    def _remove_raw_chain(self, chain_name):
-        self.iptables.ipv4['raw'].remove_chain(chain_name)
-        self.iptables.ipv6['raw'].remove_chain(chain_name)
-
     def _remove_chain_by_name_v4v6(self, chain_name):
         self.iptables.ipv4['filter'].remove_chain(chain_name)
         self.iptables.ipv6['filter'].remove_chain(chain_name)
@@ -303,9 +282,6 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._remove_rule_from_chain_v4v6('FORWARD', jump_rule, jump_rule)
 
         if direction == firewall.EGRESS_DIRECTION:
-            jump_rule = ['-m physdev --%s %s --physdev-is-bridged '
-                         '-j ACCEPT' % (self.IPTABLES_DIRECTION[direction],
-                                        device)]
             if add:
                 self._add_rules_to_chain_v4v6('INPUT', jump_rule, jump_rule,
                                               comment=ic.PORT_SEC_ACCEPT)
@@ -343,6 +319,43 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._add_rules_to_chain_v4v6('INPUT', jump_rule, jump_rule,
                                           comment=ic.INPUT_TO_SG)
 
+    def _get_br_device_name(self, port):
+        return ('brq' + port['network_id'])[:n_const.LINUX_DEV_LEN]
+
+    def _get_jump_rules(self, port):
+        zone = self.ipconntrack.get_device_zone(port)
+        br_dev = self._get_br_device_name(port)
+        port_dev = self._get_device_name(port)
+        # match by interface for bridge input
+        match_interface = '-i %s'
+        match_physdev = '-m physdev --physdev-in %s'
+        # comment to prevent duplicate warnings for different devices using
+        # same bridge. truncate start to remove prefixes
+        comment = '-m comment --comment "Set zone for %s"' % port['device'][4:]
+        rules = []
+        for dev, match in ((br_dev, match_physdev), (br_dev, match_interface),
+                           (port_dev, match_physdev)):
+            match = match % dev
+            rule = '%s %s -j CT --zone %s' % (match, comment, zone)
+            rules.append(rule)
+        return rules
+
+    def _add_conntrack_jump(self, port):
+        for jump_rule in self._get_jump_rules(port):
+            self._add_raw_rule('PREROUTING', jump_rule)
+
+    def _remove_conntrack_jump(self, port):
+        for jump_rule in self._get_jump_rules(port):
+            self._remove_raw_rule('PREROUTING', jump_rule)
+
+    def _add_raw_rule(self, chain, rule, comment=None):
+        self.iptables.ipv4['raw'].add_rule(chain, rule, comment=comment)
+        self.iptables.ipv6['raw'].add_rule(chain, rule, comment=comment)
+
+    def _remove_raw_rule(self, chain, rule):
+        self.iptables.ipv4['raw'].remove_rule(chain, rule)
+        self.iptables.ipv6['raw'].remove_rule(chain, rule)
+
     def _split_sgr_by_ethertype(self, security_group_rules):
         ipv4_sg_rules = []
         ipv6_sg_rules = []
@@ -440,23 +453,23 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         ipv6_rules += [comment_rule('-p ipv6-icmp -j RETURN',
                                     comment=ic.IPV6_ICMP_ALLOW)]
         ipv6_rules += [comment_rule('-p udp -m udp --sport 546 '
-                                    '-m udp --dport 547 '
+                                    '--dport 547 '
                                     '-j RETURN', comment=ic.DHCP_CLIENT)]
 
     def _drop_dhcp_rule(self, ipv4_rules, ipv6_rules):
         #Note(nati) Drop dhcp packet from VM
         ipv4_rules += [comment_rule('-p udp -m udp --sport 67 '
-                                    '-m udp --dport 68 '
+                                    '--dport 68 '
                                     '-j DROP', comment=ic.DHCP_SPOOF)]
         ipv6_rules += [comment_rule('-p udp -m udp --sport 547 '
-                                    '-m udp --dport 546 '
+                                    '--dport 546 '
                                     '-j DROP', comment=ic.DHCP_SPOOF)]
 
     def _accept_inbound_icmpv6(self):
         # Allow multicast listener, neighbor solicitation and
         # neighbor advertisement into the instance
         icmpv6_rules = []
-        for icmp6_type in firewall.ICMPV6_ALLOWED_TYPES:
+        for icmp6_type in firewall.ICMPV6_ALLOWED_INGRESS_TYPES:
             icmpv6_rules += ['-p ipv6-icmp -m icmp6 --icmpv6-type %s '
                              '-j RETURN' % icmp6_type]
         return icmpv6_rules
@@ -556,7 +569,9 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         return args
 
     def _generate_protocol_and_port_args(self, sg_rule):
-        args = self._protocol_arg(sg_rule.get('protocol'))
+        is_port = (sg_rule.get('source_port_range_min') is not None or
+                   sg_rule.get('port_range_min') is not None)
+        args = self._protocol_arg(sg_rule.get('protocol'), is_port)
         args += self._port_arg('sport',
                                sg_rule.get('protocol'),
                                sg_rule.get('source_port_range_min'),
@@ -618,24 +633,26 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             comment=ic.ALLOW_ASSOC)]
         return iptables_rules
 
-    def _protocol_arg(self, protocol):
-        if not protocol:
-            return []
-        if protocol == 'icmpv6':
-            protocol = 'ipv6-icmp'
-        iptables_rule = ['-p', protocol]
+    def _protocol_arg(self, protocol, is_port):
+        iptables_rule = []
+        rule_protocol = n_const.IPTABLES_PROTOCOL_NAME_MAP.get(protocol,
+                                                               protocol)
+        # protocol zero is a special case and requires no '-p'
+        if rule_protocol:
+            iptables_rule = ['-p', rule_protocol]
+
+            if (is_port and rule_protocol in constants.IPTABLES_PROTOCOL_MAP):
+                # iptables adds '-m protocol' when the port number is specified
+                iptables_rule += ['-m',
+                    constants.IPTABLES_PROTOCOL_MAP[rule_protocol]]
         return iptables_rule
 
     def _port_arg(self, direction, protocol, port_range_min, port_range_max):
-        if (protocol not in ['udp', 'tcp', 'icmp', 'ipv6-icmp']
-            or port_range_min is None):
-            return []
-
-        protocol_modules = {'udp': 'udp', 'tcp': 'tcp',
-                            'icmp': 'icmp', 'ipv6-icmp': 'icmp6'}
-        # iptables adds '-m protocol' when the port number is specified
-        args = ['-m', protocol_modules[protocol]]
+        args = []
+        if port_range_min is None:
+            return args
 
+        protocol = n_const.IPTABLES_PROTOCOL_NAME_MAP.get(protocol, protocol)
         if protocol in ['icmp', 'ipv6-icmp']:
             protocol_type = 'icmpv6' if protocol == 'ipv6-icmp' else 'icmp'
             # Note(xuhanp): port_range_min/port_range_max represent
@@ -690,7 +707,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         remote_sgs_to_remove = self._determine_remote_sgs_to_remove(
             filtered_ports)
 
-        for ip_version, remote_sg_ids in six.iteritems(remote_sgs_to_remove):
+        for ip_version, remote_sg_ids in remote_sgs_to_remove.items():
             if self.enable_ipset:
                 self._remove_ipsets_for_remote_sgs(ip_version, remote_sg_ids)
 
@@ -710,8 +727,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
                                        constants.IPv6: set()}
         remote_group_id_sets = self._get_remote_sg_ids_sets_by_ipversion(
             filtered_ports)
-        for ip_version, remote_group_id_set in (
-                six.iteritems(remote_group_id_sets)):
+        for ip_version, remote_group_id_set in remote_group_id_sets.items():
             sgs_to_remove_per_ipversion[ip_version].update(
                 set(self.pre_sg_members) - remote_group_id_set)
         return sgs_to_remove_per_ipversion
@@ -771,7 +787,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
 
     def _clean_deleted_sg_rule_conntrack_entries(self):
         deleted_sg_ids = set()
-        for sg_id in self.updated_rule_sg_ids:
+        for sg_id in set(self.updated_rule_sg_ids):
             del_rules = self._find_deleted_sg_rules(sg_id)
             if not del_rules:
                 continue
@@ -785,7 +801,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
 
     def _clean_updated_sg_member_conntrack_entries(self):
         updated_device_ids = set()
-        for device in self.updated_sg_members:
+        for device in set(self.updated_sg_members):
             sec_group_change = False
             device_info = self.filtered_ports.get(device)
             pre_device_info = self._pre_defer_filtered_ports.get(device)
@@ -823,7 +839,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
     def _remove_conntrack_entries_from_sg_updates(self):
         self._clean_deleted_sg_rule_conntrack_entries()
         self._clean_updated_sg_member_conntrack_entries()
-        self._clean_deleted_remote_sg_members_conntrack_entries()
+        if not self.enable_ipset:
+            self._clean_deleted_remote_sg_members_conntrack_entries()
 
     def _get_sg_members(self, sg_info, sg_id, ethertype):
         return set(sg_info.get(sg_id, {}).get(ethertype, []))
@@ -841,116 +858,17 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._pre_defer_filtered_ports = None
             self._pre_defer_unfiltered_ports = None
 
-    def _populate_initial_zone_map(self):
-        """Setup the map between devices and zones based on current rules."""
-        self._device_zone_map = {}
-        rules = self.iptables.get_rules_for_table('raw')
-        for rule in rules:
-            match = re.match(r'.* --physdev-in (?P<dev>[a-zA-Z0-9\-]+)'
-                             r'.* -j CT --zone (?P<zone>\d+).*', rule)
-            if match:
-                # strip off any prefix that the interface is using
-                short_port_id = match.group('dev')[LINUX_DEV_PREFIX_LEN:]
-                self._device_zone_map[short_port_id] = int(match.group('zone'))
-        LOG.debug("Populated conntrack zone map: %s", self._device_zone_map)
-
-    def get_device_zone(self, port_id):
-        # we have to key the device_zone_map based on the fragment of the port
-        # UUID that shows up in the interface name. This is because the initial
-        # map is populated strictly based on interface names that we don't know
-        # the full UUID of.
-        short_port_id = port_id[:(LINUX_DEV_LEN - LINUX_DEV_PREFIX_LEN)]
-        try:
-            return self._device_zone_map[short_port_id]
-        except KeyError:
-            return self._generate_device_zone(short_port_id)
-
-    def _free_zones_from_removed_ports(self):
-        """Clears any entries from the zone map of removed ports."""
-        existing_ports = [
-            port['device'][:(LINUX_DEV_LEN - LINUX_DEV_PREFIX_LEN)]
-            for port in (list(self.filtered_ports.values()) +
-                         list(self.unfiltered_ports.values()))
-        ]
-        removed = set(self._device_zone_map) - set(existing_ports)
-        for dev in removed:
-            self._device_zone_map.pop(dev, None)
-
-    def _generate_device_zone(self, short_port_id):
-        """Generates a unique conntrack zone for the passed in ID."""
-        try:
-            zone = self._find_open_zone()
-        except n_exc.CTZoneExhaustedError:
-            # Free some zones and try again, repeat failure will not be caught
-            self._free_zones_from_removed_ports()
-            zone = self._find_open_zone()
-
-        self._device_zone_map[short_port_id] = zone
-        LOG.debug("Assigned CT zone %(z)s to port %(dev)s.",
-                  {'z': zone, 'dev': short_port_id})
-        return self._device_zone_map[short_port_id]
-
-    def _find_open_zone(self):
-        # call set to dedup because old ports may be mapped to the same zone.
-        zones_in_use = sorted(set(self._device_zone_map.values()))
-        if not zones_in_use:
-            return 1
-        # attempt to increment onto the highest used zone first. if we hit the
-        # end, go back and look for any gaps left by removed devices.
-        last = zones_in_use[-1]
-        if last < MAX_CONNTRACK_ZONES:
-            return last + 1
-        for index, used in enumerate(zones_in_use):
-            if used - index != 1:
-                # gap found, let's use it!
-                return index + 1
-        # conntrack zones exhausted :( :(
-        raise n_exc.CTZoneExhaustedError()
-
 
 class OVSHybridIptablesFirewallDriver(IptablesFirewallDriver):
-    OVS_HYBRID_TAP_PREFIX = constants.TAP_DEVICE_PREFIX
     OVS_HYBRID_PLUG_REQUIRED = True
+    CONNTRACK_ZONE_PER_PORT = True
 
     def _port_chain_name(self, port, direction):
         return iptables_manager.get_chain_name(
             '%s%s' % (CHAIN_NAME_PREFIX[direction], port['device']))
 
     def _get_br_device_name(self, port):
-        return ('qvb' + port['device'])[:LINUX_DEV_LEN]
+        return ('qvb' + port['device'])[:n_const.LINUX_DEV_LEN]
 
     def _get_device_name(self, port):
         return get_hybrid_port_name(port['device'])
-
-    def _get_jump_rule(self, port, direction):
-        if direction == firewall.INGRESS_DIRECTION:
-            device = self._get_br_device_name(port)
-        else:
-            device = self._get_device_name(port)
-        jump_rule = '-m physdev --physdev-in %s -j CT --zone %s' % (
-            device, self.get_device_zone(port['device']))
-        return jump_rule
-
-    def _add_raw_chain_rules(self, port, direction):
-        jump_rule = self._get_jump_rule(port, direction)
-        self.iptables.ipv4['raw'].add_rule('PREROUTING', jump_rule)
-        self.iptables.ipv6['raw'].add_rule('PREROUTING', jump_rule)
-
-    def _remove_raw_chain_rules(self, port, direction):
-        jump_rule = self._get_jump_rule(port, direction)
-        self.iptables.ipv4['raw'].remove_rule('PREROUTING', jump_rule)
-        self.iptables.ipv6['raw'].remove_rule('PREROUTING', jump_rule)
-
-    def _add_chain(self, port, direction):
-        super(OVSHybridIptablesFirewallDriver, self)._add_chain(port,
-                                                                direction)
-        if direction in [firewall.INGRESS_DIRECTION,
-                         firewall.EGRESS_DIRECTION]:
-            self._add_raw_chain_rules(port, direction)
-
-    def _remove_chain(self, port, direction):
-        super(OVSHybridIptablesFirewallDriver, self)._remove_chain(port,
-                                                                   direction)
-        if direction in [firewall.INGRESS_DIRECTION,
-                         firewall.EGRESS_DIRECTION]:
-            self._remove_raw_chain_rules(port, direction)
