@@ -17,6 +17,7 @@ import collections
 
 from neutron_lib import exceptions
 from oslo_log import helpers as log_helpers
+from oslo_log import log as logging
 import oslo_messaging
 
 from neutron._i18n import _
@@ -30,6 +31,8 @@ from neutron.common import rpc as n_rpc
 from neutron.common import topics
 from neutron.objects import base as obj_base
 
+LOG = logging.getLogger(__name__)
+
 
 class ResourcesRpcError(exceptions.NeutronException):
     pass
@@ -49,6 +52,14 @@ def _validate_resource_type(resource_type):
         raise InvalidResourceTypeClass(resource_type=resource_type)
 
 
+def _resource_to_class(resource_type):
+    _validate_resource_type(resource_type)
+
+    # we've already validated the resource type, so we are pretty sure the
+    # class is there => no need to validate it specifically
+    return resources.get_resource_cls(resource_type)
+
+
 def resource_type_versioned_topic(resource_type, version=None):
     """Return the topic for a resource type.
 
@@ -74,19 +85,14 @@ class ResourcesPullRpcApi(object):
         if not hasattr(cls, '_instance'):
             cls._instance = super(ResourcesPullRpcApi, cls).__new__(cls)
             target = oslo_messaging.Target(
-                topic=topics.PLUGIN, version='1.0',
+                topic=topics.PLUGIN, version='1.1',
                 namespace=constants.RPC_NAMESPACE_RESOURCES)
             cls._instance.client = n_rpc.get_client(target)
         return cls._instance
 
     @log_helpers.log_method_call
     def pull(self, context, resource_type, resource_id):
-        _validate_resource_type(resource_type)
-
-        # we've already validated the resource type, so we are pretty sure the
-        # class is there => no need to validate it specifically
-        resource_type_cls = resources.get_resource_cls(resource_type)
-
+        resource_type_cls = _resource_to_class(resource_type)
         cctxt = self.client.prepare()
         primitive = cctxt.call(context, 'pull',
             resource_type=resource_type,
@@ -95,9 +101,18 @@ class ResourcesPullRpcApi(object):
         if primitive is None:
             raise ResourceNotFound(resource_type=resource_type,
                                    resource_id=resource_id)
-
         return resource_type_cls.clean_obj_from_primitive(primitive)
 
+    @log_helpers.log_method_call
+    def bulk_pull(self, context, resource_type, filter_kwargs=None):
+        resource_type_cls = _resource_to_class(resource_type)
+        cctxt = self.client.prepare()
+        primitives = cctxt.call(context, 'bulk_pull',
+            resource_type=resource_type,
+            version=resource_type_cls.VERSION, filter_kwargs=filter_kwargs)
+        return [resource_type_cls.clean_obj_from_primitive(primitive)
+                for primitive in primitives]
+
 
 class ResourcesPullRpcCallback(object):
     """Plugin-side RPC (implementation) for agent-to-plugin interaction.
@@ -109,9 +124,10 @@ class ResourcesPullRpcCallback(object):
 
     # History
     #   1.0 Initial version
+    #   1.1 Added bulk_pull
 
     target = oslo_messaging.Target(
-        version='1.0', namespace=constants.RPC_NAMESPACE_RESOURCES)
+        version='1.1', namespace=constants.RPC_NAMESPACE_RESOURCES)
 
     @oslo_messaging.expected_exceptions(rpc_exc.CallbackNotFound)
     def pull(self, context, resource_type, version, resource_id):
@@ -119,6 +135,16 @@ class ResourcesPullRpcCallback(object):
         if obj:
             return obj.obj_to_primitive(target_version=version)
 
+    @oslo_messaging.expected_exceptions(rpc_exc.CallbackNotFound)
+    def bulk_pull(self, context, resource_type, version, filter_kwargs=None):
+        filter_kwargs = filter_kwargs or {}
+        resource_type_cls = _resource_to_class(resource_type)
+        # TODO(kevinbenton): add in producer registry so producers can add
+        # hooks to mangle these things like they can with 'pull'.
+        return [obj.obj_to_primitive(target_version=version)
+                for obj in resource_type_cls.get_objects(context, _pager=None,
+                                                         **filter_kwargs)]
+
 
 class ResourcesPushToServersRpcApi(object):
     """Publisher-side RPC (stub) for plugin-to-plugin fanout interaction.
@@ -198,7 +224,6 @@ class ResourcesPushRpcApi(object):
             resources_by_type[resource_type].append(resource)
         return resources_by_type
 
-    @log_helpers.log_method_call
     def push(self, context, resource_list, event_type):
         """Push an event and list of resources to agents, batched per type.
         When a list of different resource types is passed to this method,
@@ -207,35 +232,31 @@ class ResourcesPushRpcApi(object):
         """
 
         resources_by_type = self._classify_resources_by_type(resource_list)
+        LOG.debug(
+            "Pushing event %s for resources: %s", event_type,
+            {t: ["ID=%s,revision_number=%s" % (
+                     getattr(obj, 'id', None),
+                     getattr(obj, 'revision_number', None))
+                 for obj in resources_by_type[t]]
+             for t in resources_by_type})
         for resource_type, type_resources in resources_by_type.items():
             self._push(context, resource_type, type_resources, event_type)
 
     def _push(self, context, resource_type, resource_list, event_type):
         """Push an event and list of resources of the same type to agents."""
         _validate_resource_type(resource_type)
-        compat_call = len(resource_list) == 1
 
         for version in version_manager.get_resource_versions(resource_type):
             cctxt = self._prepare_object_fanout_context(
-                resource_list[0], version,
-                rpc_version='1.0' if compat_call else '1.1')
+                resource_list[0], version, rpc_version='1.1')
 
             dehydrated_resources = [
                 resource.obj_to_primitive(target_version=version)
                 for resource in resource_list]
 
-            if compat_call:
-                #TODO(mangelajo): remove in Ocata, backwards compatibility
-                #                 for agents expecting a single element as
-                #                 a single element instead of a list, this
-                #                 is only relevant to the QoSPolicy topic queue
-                cctxt.cast(context, 'push',
-                           resource=dehydrated_resources[0],
-                           event_type=event_type)
-            else:
-                cctxt.cast(context, 'push',
-                           resource_list=dehydrated_resources,
-                           event_type=event_type)
+            cctxt.cast(context, 'push',
+                       resource_list=dehydrated_resources,
+                       event_type=event_type)
 
 
 class ResourcesPushRpcCallback(object):
@@ -255,10 +276,7 @@ class ResourcesPushRpcCallback(object):
     @oslo_messaging.expected_exceptions(rpc_exc.CallbackNotFound)
     def push(self, context, **kwargs):
         """Push receiver, will always receive resources of the same type."""
-        # TODO(mangelajo): accept single 'resource' parameter for backwards
-        #                  compatibility during Newton, remove in Ocata
-        resource_list = ([kwargs['resource']] if 'resource' in kwargs else
-                         kwargs['resource_list'])
+        resource_list = kwargs['resource_list']
         event_type = kwargs['event_type']
 
         resource_objs = [
@@ -266,4 +284,4 @@ class ResourcesPushRpcCallback(object):
             for resource in resource_list]
 
         resource_type = resources.get_resource_type(resource_objs[0])
-        cons_registry.push(resource_type, resource_objs, event_type)
+        cons_registry.push(context, resource_type, resource_objs, event_type)
