@@ -16,6 +16,7 @@
 import copy
 
 import netaddr
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 from oslo_db import exception as db_exc
@@ -23,14 +24,14 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from sqlalchemy import and_
 
-from neutron._i18n import _, _LE, _LW
-from neutron.common import constants as n_const
 from neutron.common import ipv6_utils
+from neutron.db import api as db_api
 from neutron.db import ipam_backend_mixin
 from neutron.db import models_v2
-from neutron.extensions import portbindings
 from neutron.ipam import driver
 from neutron.ipam import exceptions as ipam_exc
+from neutron.objects import ports as port_obj
+from neutron.objects import subnet as obj_subnet
 
 
 LOG = logging.getLogger(__name__)
@@ -52,7 +53,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         try:
             func(*args, **kwargs)
         except Exception as e:
-            LOG.warning(_LW("Revert failed with: %s"), e)
+            LOG.warning("Revert failed with: %s", e)
 
     def _ipam_deallocate_ips(self, context, ipam_driver, port, ips,
                              revert_on_fail=True):
@@ -89,8 +90,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                 elif not revert_on_fail and ips:
                     addresses = ', '.join(self._get_failed_ips(ips,
                                                                deallocated))
-                    LOG.error(_LE("IP deallocation failed on "
-                                  "external system for %s"), addresses)
+                    LOG.error("IP deallocation failed on "
+                              "external system for %s", addresses)
         return deallocated
 
     def _ipam_allocate_ips(self, context, ipam_driver, port, ips,
@@ -143,8 +144,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                 elif not revert_on_fail and ips:
                     addresses = ', '.join(self._get_failed_ips(ips,
                                                                allocated))
-                    LOG.error(_LE("IP allocation failed on "
-                                  "external system for %s"), addresses)
+                    LOG.error("IP allocation failed on "
+                              "external system for %s", addresses)
 
         return allocated
 
@@ -198,15 +199,16 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         a subnet_id then allocate an IP address accordingly.
         """
         p = port['port']
+        fixed_configured = p['fixed_ips'] is not constants.ATTR_NOT_SPECIFIED
         subnets = self._ipam_get_subnets(context,
                                          network_id=p['network_id'],
                                          host=p.get(portbindings.HOST_ID),
-                                         service_type=p.get('device_owner'))
+                                         service_type=p.get('device_owner'),
+                                         fixed_configured=fixed_configured)
 
         v4, v6_stateful, v6_stateless = self._classify_subnets(
             context, subnets)
 
-        fixed_configured = p['fixed_ips'] is not constants.ATTR_NOT_SPECIFIED
         if fixed_configured:
             ips = self._test_fixed_ips_for_port(context,
                                                 p["network_id"],
@@ -221,19 +223,28 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     ips.append([{'subnet_id': s['id']}
                                 for s in subnets])
 
-        is_router_port = (
-            p['device_owner'] in constants.ROUTER_INTERFACE_OWNERS_SNAT)
-        if not is_router_port:
-            for subnet in v6_stateless:
-                # IP addresses for IPv6 SLAAC and DHCPv6-stateless subnets
-                # are implicitly included.
-                ips.append({'subnet_id': subnet['id'],
-                            'subnet_cidr': subnet['cidr'],
-                            'eui64_address': True,
-                            'mac': p['mac_address']})
+        ips.extend(self._get_auto_address_ips(v6_stateless, p))
+
         ipam_driver = driver.Pool.get_instance(None, context)
         return self._ipam_allocate_ips(context, ipam_driver, p, ips)
 
+    def _get_auto_address_ips(self, v6_stateless_subnets, port,
+                              exclude_subnet_ids=None):
+        exclude_subnet_ids = exclude_subnet_ids or []
+        ips = []
+        is_router_port = (
+            port['device_owner'] in constants.ROUTER_INTERFACE_OWNERS_SNAT)
+        if not is_router_port:
+            for subnet in v6_stateless_subnets:
+                if subnet['id'] not in exclude_subnet_ids:
+                    # IP addresses for IPv6 SLAAC and DHCPv6-stateless subnets
+                    # are implicitly included.
+                    ips.append({'subnet_id': subnet['id'],
+                                'subnet_cidr': subnet['cidr'],
+                                'eui64_address': True,
+                                'mac': port['mac_address']})
+        return ips
+
     def _test_fixed_ips_for_port(self, context, network_id, fixed_ips,
                                  device_owner, subnets):
         """Test fixed IPs for port.
@@ -251,15 +262,11 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
 
             is_auto_addr_subnet = ipv6_utils.is_auto_address_subnet(subnet)
             if ('ip_address' in fixed and
-                    subnet['cidr'] != n_const.PROVISIONAL_IPV6_PD_PREFIX):
+                    subnet['cidr'] != constants.PROVISIONAL_IPV6_PD_PREFIX):
                 if (is_auto_addr_subnet and device_owner not in
                         constants.ROUTER_INTERFACE_OWNERS):
-                    msg = (_("IPv6 address %(address)s can not be directly "
-                            "assigned to a port on subnet %(id)s since the "
-                            "subnet is configured for automatic addresses") %
-                           {'address': fixed['ip_address'],
-                            'id': subnet['id']})
-                    raise n_exc.InvalidInput(error_message=msg)
+                    raise ipam_exc.AllocationOnAutoAddressSubnet(
+                        ip=fixed['ip_address'], subnet_id=subnet['id'])
                 fixed_ip_list.append({'subnet_id': subnet['id'],
                                       'ip_address': fixed['ip_address']})
             else:
@@ -271,7 +278,6 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                         not is_auto_addr_subnet):
                     fixed_ip_list.append({'subnet_id': subnet['id']})
 
-        self._validate_max_ips_per_port(fixed_ip_list, device_owner)
         return fixed_ip_list
 
     def _update_ips_for_port(self, context, port, host,
@@ -283,7 +289,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
             context, original_ips, new_ips, port['device_owner'])
         try:
             subnets = self._ipam_get_subnets(
-                context, network_id=port['network_id'], host=host)
+                context, network_id=port['network_id'], host=host,
+                service_type=port.get('device_owner'))
         except ipam_exc.DeferIpam:
             subnets = []
 
@@ -300,6 +307,14 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         if changes.remove:
             removed = self._ipam_deallocate_ips(context, ipam_driver, port,
                                                 changes.remove)
+
+        v6_stateless = self._classify_subnets(
+            context, subnets)[2]
+        handled_subnet_ids = [ip['subnet_id'] for ip in
+                              to_add + changes.original + changes.remove]
+        to_add.extend(self._get_auto_address_ips(
+            v6_stateless, port, handled_subnet_ids))
+
         if to_add:
             added = self._ipam_allocate_ips(context, ipam_driver,
                                             port, to_add)
@@ -307,18 +322,37 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                             original=changes.original,
                             remove=removed)
 
+    @db_api.context_manager.writer
     def save_allocation_pools(self, context, subnet, allocation_pools):
         for pool in allocation_pools:
             first_ip = str(netaddr.IPAddress(pool.first, pool.version))
             last_ip = str(netaddr.IPAddress(pool.last, pool.version))
-            ip_pool = models_v2.IPAllocationPool(subnet=subnet,
-                                                 first_ip=first_ip,
-                                                 last_ip=last_ip)
-            context.session.add(ip_pool)
+            obj_subnet.IPAllocationPool(
+                context, subnet_id=subnet['id'], start=first_ip,
+                end=last_ip).create()
 
     def update_port_with_ips(self, context, host, db_port, new_port, new_mac):
         changes = self.Changes(add=[], original=[], remove=[])
 
+        auto_assign_subnets = []
+        if new_mac:
+            original = self._make_port_dict(db_port, process_extensions=False)
+            if original.get('mac_address') != new_mac:
+                original_ips = original.get('fixed_ips', [])
+                new_ips = new_port.setdefault('fixed_ips', original_ips)
+                new_ips_subnets = [new_ip['subnet_id'] for new_ip in new_ips]
+                for orig_ip in original_ips:
+                    if ipv6_utils.is_eui64_address(orig_ip.get('ip_address')):
+                        subnet_to_delete = {}
+                        subnet_to_delete['subnet_id'] = orig_ip['subnet_id']
+                        subnet_to_delete['delete_subnet'] = True
+                        auto_assign_subnets.append(subnet_to_delete)
+                        try:
+                            i = new_ips_subnets.index(orig_ip['subnet_id'])
+                            new_ips[i] = subnet_to_delete
+                        except ValueError:
+                            new_ips.append(subnet_to_delete)
+
         if 'fixed_ips' in new_port:
             original = self._make_port_dict(db_port,
                                             process_extensions=False)
@@ -345,6 +379,16 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     ip['subnet_id'], db_port.id)
             self._update_db_port(context, db_port, new_port, network_id,
                                  new_mac)
+
+            if auto_assign_subnets:
+                port_copy = copy.deepcopy(original)
+                port_copy.update(new_port)
+                port_copy['fixed_ips'] = auto_assign_subnets
+                self.allocate_ips_for_port_and_store(context,
+                            {'port': port_copy}, port_copy['id'])
+
+            getattr(db_port, 'fixed_ips')  # refresh relationship before return
+
         except Exception:
             with excutils.save_and_reraise_exception():
                 if 'fixed_ips' in new_port:
@@ -404,6 +448,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
 
     def add_auto_addrs_on_network_ports(self, context, subnet, ipam_subnet):
         """For an auto-address subnet, add addrs for ports on the net."""
+        # TODO(ataraday): switched for writer when flush_on_subtransaction
+        # will be available for neutron
         with context.session.begin(subtransactions=True):
             network_id = subnet['network_id']
             port_qry = context.session.query(models_v2.Port)
@@ -420,18 +466,18 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                       'eui64_address': True,
                       'mac': port['mac_address']}
                 ip_request = factory.get_request(context, port, ip)
-                ip_address = ipam_subnet.allocate(ip_request)
-                allocated = models_v2.IPAllocation(network_id=network_id,
-                                                   port_id=port['id'],
-                                                   ip_address=ip_address,
-                                                   subnet_id=subnet['id'])
                 try:
+                    ip_address = ipam_subnet.allocate(ip_request)
+                    allocated = port_obj.IPAllocation(
+                        context, network_id=network_id, port_id=port['id'],
+                        ip_address=ip_address, subnet_id=subnet['id'])
                     # Do the insertion of each IP allocation entry within
                     # the context of a nested transaction, so that the entry
                     # is rolled back independently of other entries whenever
-                    # the corresponding port has been deleted.
-                    with context.session.begin_nested():
-                        context.session.add(allocated)
+                    # the corresponding port has been deleted; since OVO
+                    # already opens a nested transaction, we don't need to do
+                    # it explicitly here.
+                    allocated.create()
                     updated_ports.append(port['id'])
                 except db_exc.DBReferenceError:
                     LOG.debug("Port %s was deleted while updating it with an "
@@ -443,6 +489,10 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     except Exception:
                         LOG.debug("Reverting IP allocation failed for %s",
                                   ip_address)
+                except ipam_exc.IpAddressAlreadyAllocated:
+                    LOG.debug("Port %s got IPv6 auto-address in a concurrent "
+                              "create or update port request. Ignoring.",
+                              port['id'])
             return updated_ports
 
     def allocate_subnet(self, context, network, subnet, subnetpool_id):
