@@ -13,52 +13,45 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api.definitions import port as port_def
+from neutron_lib.api.definitions import portbindings
 from neutron_lib.api import validators
-from neutron_lib.db import model_base
-import sqlalchemy as sa
-from sqlalchemy import orm
+from neutron_lib.plugins import directory
 
-from neutron.api.v2 import attributes
-from neutron.db import db_base_plugin_v2
+from neutron.db import _model_query as model_query
+from neutron.db import _resource_extend as resource_extend
+from neutron.db import api as db_api
+from neutron.db.models import portbinding as pmodels
 from neutron.db import models_v2
 from neutron.db import portbindings_base
-from neutron.extensions import portbindings
 
 
-class PortBindingPort(model_base.BASEV2):
-    port_id = sa.Column(sa.String(36),
-                        sa.ForeignKey('ports.id', ondelete="CASCADE"),
-                        primary_key=True)
-    host = sa.Column(sa.String(255), nullable=False)
-    port = orm.relationship(
-        models_v2.Port,
-        backref=orm.backref("portbinding",
-                            lazy='joined', uselist=False,
-                            cascade='delete'))
+def _port_model_hook(context, original_model, query):
+    query = query.outerjoin(
+        pmodels.PortBindingPort,
+        (original_model.id == pmodels.PortBindingPort.port_id))
+    return query
 
 
+def _port_result_filter_hook(query, filters):
+    values = filters and filters.get(portbindings.HOST_ID, [])
+    if not values:
+        return query
+    query = query.filter(pmodels.PortBindingPort.host.in_(values))
+    return query
+
+
+@resource_extend.has_resource_extenders
 class PortBindingMixin(portbindings_base.PortBindingBaseMixin):
-    extra_binding_dict = None
 
-    def _port_model_hook(self, context, original_model, query):
-        query = query.outerjoin(PortBindingPort,
-                                (original_model.id ==
-                                 PortBindingPort.port_id))
-        return query
-
-    def _port_result_filter_hook(self, query, filters):
-        values = filters and filters.get(portbindings.HOST_ID, [])
-        if not values:
-            return query
-        query = query.filter(PortBindingPort.host.in_(values))
-        return query
-
-    db_base_plugin_v2.NeutronDbPluginV2.register_model_query_hook(
-        models_v2.Port,
-        "portbindings_port",
-        '_port_model_hook',
-        None,
-        '_port_result_filter_hook')
+    def __new__(cls, *args, **kwargs):
+        model_query.register_hook(
+            models_v2.Port,
+            "portbindings_port",
+            query_hook=_port_model_hook,
+            filter_hook=None,
+            result_filters=_port_result_filter_hook)
+        return super(PortBindingMixin, cls).__new__(cls, *args, **kwargs)
 
     def _process_portbindings_create_and_update(self, context, port_data,
                                                 port):
@@ -78,13 +71,13 @@ class PortBindingMixin(portbindings_base.PortBindingBaseMixin):
 
         host = port_data.get(portbindings.HOST_ID)
         host_set = validators.is_attr_set(host)
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             bind_port = context.session.query(
-                PortBindingPort).filter_by(port_id=port['id']).first()
+                pmodels.PortBindingPort).filter_by(port_id=port['id']).first()
             if host_set:
                 if not bind_port:
-                    context.session.add(PortBindingPort(port_id=port['id'],
-                                                        host=host))
+                    context.session.add(
+                        pmodels.PortBindingPort(port_id=port['id'], host=host))
                 else:
                     bind_port.host = host
             else:
@@ -92,9 +85,12 @@ class PortBindingMixin(portbindings_base.PortBindingBaseMixin):
         self._extend_port_dict_binding_host(port, host)
 
     def get_port_host(self, context, port_id):
-        with context.session.begin(subtransactions=True):
-            bind_port = context.session.query(
-                PortBindingPort).filter_by(port_id=port_id).first()
+        with db_api.context_manager.reader.using(context):
+            bind_port = (
+                context.session.query(pmodels.PortBindingPort).
+                filter_by(port_id=port_id).
+                first()
+            )
             return bind_port.host if bind_port else None
 
     def _extend_port_dict_binding_host(self, port_res, host):
@@ -106,13 +102,10 @@ class PortBindingMixin(portbindings_base.PortBindingBaseMixin):
         host = port_db.portbinding.host if port_db.portbinding else None
         self._extend_port_dict_binding_host(port_res, host)
 
-
-def _extend_port_dict_binding(plugin, port_res, port_db):
-    if not isinstance(plugin, PortBindingMixin):
-        return
-    plugin.extend_port_dict_binding(port_res, port_db)
-
-
-# Register dict extend functions for ports
-db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
-    attributes.PORTS, [_extend_port_dict_binding])
+    @staticmethod
+    @resource_extend.extends([port_def.COLLECTION_NAME])
+    def _extend_port_dict_binding(port_res, port_db):
+        plugin = directory.get_plugin()
+        if not isinstance(plugin, PortBindingMixin):
+            return
+        plugin.extend_port_dict_binding(port_res, port_db)
