@@ -12,18 +12,18 @@
 
 import netaddr
 
-from oslo_versionedobjects import base as obj_base
 from oslo_versionedobjects import fields as obj_fields
 
 from neutron.common import utils
+from neutron.db.models import subnet_service_type
 from neutron.db import models_v2
-from neutron.db import rbac_db_models
 from neutron.objects import base
 from neutron.objects import common_types
+from neutron.objects import network
 from neutron.objects import rbac_db
 
 
-@obj_base.VersionedObjectRegistry.register
+@base.NeutronObjectRegistry.register
 class DNSNameServer(base.NeutronDbObject):
     # Version 1.0: Initial version
     VERSION = '1.0'
@@ -36,7 +36,7 @@ class DNSNameServer(base.NeutronDbObject):
 
     fields = {
         'address': obj_fields.StringField(),
-        'subnet_id': obj_fields.UUIDField(),
+        'subnet_id': common_types.UUIDField(),
         'order': obj_fields.IntegerField()
     }
 
@@ -55,7 +55,7 @@ class DNSNameServer(base.NeutronDbObject):
                                                      **kwargs)
 
 
-@obj_base.VersionedObjectRegistry.register
+@base.NeutronObjectRegistry.register
 class Route(base.NeutronDbObject):
     # Version 1.0: Initial version
     VERSION = '1.0'
@@ -67,7 +67,7 @@ class Route(base.NeutronDbObject):
     foreign_keys = {'Subnet': {'subnet_id': 'id'}}
 
     fields = {
-        'subnet_id': obj_fields.UUIDField(),
+        'subnet_id': common_types.UUIDField(),
         'destination': common_types.IPNetworkField(),
         'nexthop': obj_fields.IPAddressField()
     }
@@ -94,7 +94,7 @@ class Route(base.NeutronDbObject):
         return result
 
 
-@obj_base.VersionedObjectRegistry.register
+@base.NeutronObjectRegistry.register
 class IPAllocationPool(base.NeutronDbObject):
     # Version 1.0: Initial version
     VERSION = '1.0'
@@ -109,12 +109,14 @@ class IPAllocationPool(base.NeutronDbObject):
     }
 
     fields = {
-        'id': obj_fields.UUIDField(),
-        'subnet_id': obj_fields.UUIDField(),
+        'id': common_types.UUIDField(),
+        'subnet_id': common_types.UUIDField(),
         'start': obj_fields.IPAddressField(),
         'end': obj_fields.IPAddressField()
     }
 
+    fields_no_update = ['subnet_id']
+
     @classmethod
     def modify_fields_from_db(cls, db_obj):
         # TODO(korzen) remove this method when IP and CIDR decorator ready
@@ -136,6 +138,23 @@ class IPAllocationPool(base.NeutronDbObject):
         return result
 
 
+@base.NeutronObjectRegistry.register
+class SubnetServiceType(base.NeutronDbObject):
+    # Version 1.0: Initial version
+    VERSION = '1.0'
+
+    db_model = subnet_service_type.SubnetServiceType
+
+    foreign_keys = {'Subnet': {'subnet_id': 'id'}}
+
+    primary_keys = ['subnet_id', 'service_type']
+
+    fields = {
+        'subnet_id': common_types.UUIDField(),
+        'service_type': obj_fields.StringField()
+    }
+
+
 # RBAC metaclass is not applied here because 'shared' attribute of Subnet
 # is dependent on Network 'shared' state, and in Subnet object
 # it can be read-only. The necessary changes are applied manually:
@@ -143,7 +162,7 @@ class IPAllocationPool(base.NeutronDbObject):
 #   - added 'shared' to synthetic_fields
 #   - registered extra_filter_name for 'shared' attribute
 #   - added loading shared attribute based on network 'rbac_entries'
-@obj_base.VersionedObjectRegistry.register
+@base.NeutronObjectRegistry.register
 class Subnet(base.NeutronDbObject):
     # Version 1.0: Initial version
     VERSION = '1.0'
@@ -151,12 +170,14 @@ class Subnet(base.NeutronDbObject):
     db_model = models_v2.Subnet
 
     fields = {
-        'id': obj_fields.UUIDField(),
+        'id': common_types.UUIDField(),
         'project_id': obj_fields.StringField(nullable=True),
         'name': obj_fields.StringField(nullable=True),
-        'network_id': obj_fields.UUIDField(),
-        'segment_id': obj_fields.UUIDField(nullable=True),
-        'subnetpool_id': obj_fields.UUIDField(nullable=True),
+        'network_id': common_types.UUIDField(),
+        'segment_id': common_types.UUIDField(nullable=True),
+        # NOTE: subnetpool_id can be 'prefix_delegation' string
+        # when the IPv6 Prefix Delegation is enabled
+        'subnetpool_id': obj_fields.StringField(nullable=True),
         'ip_version': common_types.IPVersionEnumField(),
         'cidr': common_types.IPNetworkField(),
         'gateway_ip': obj_fields.IPAddressField(nullable=True),
@@ -168,18 +189,18 @@ class Subnet(base.NeutronDbObject):
                                                          nullable=True),
         'host_routes': obj_fields.ListOfObjectsField('Route', nullable=True),
         'ipv6_ra_mode': common_types.IPV6ModeEnumField(nullable=True),
-        'ipv6_address_mode': common_types.IPV6ModeEnumField(nullable=True)
+        'ipv6_address_mode': common_types.IPV6ModeEnumField(nullable=True),
+        'service_types': obj_fields.ListOfStringsField(nullable=True)
     }
 
     synthetic_fields = ['allocation_pools', 'dns_nameservers', 'host_routes',
-                        'shared']
+                        'service_types', 'shared']
 
     foreign_keys = {'Network': {'network_id': 'id'}}
 
-    fields_no_update = ['project_id']
+    fields_no_update = ['project_id', 'network_id', 'segment_id']
 
     fields_need_translation = {
-        'project_id': 'tenant_id',
         'host_routes': 'routes'
     }
 
@@ -190,6 +211,8 @@ class Subnet(base.NeutronDbObject):
     def obj_load_attr(self, attrname):
         if attrname == 'shared':
             return self._load_shared()
+        if attrname == 'service_types':
+            return self._load_service_types()
         super(Subnet, self).obj_load_attr(attrname)
 
     def _load_shared(self, db_obj=None):
@@ -205,15 +228,27 @@ class Subnet(base.NeutronDbObject):
             # create), it should be rare case to load 'shared' by that method
             shared = (rbac_db.RbacNeutronDbObjectMixin.
                       get_shared_with_tenant(self.obj_context.elevated(),
-                                             rbac_db_models.NetworkRBAC,
+                                             network.NetworkRBAC,
                                              self.network_id,
                                              self.project_id))
         setattr(self, 'shared', shared)
         self.obj_reset_changes(['shared'])
 
+    def _load_service_types(self, db_obj=None):
+        if db_obj:
+            service_types = db_obj.get('service_types', [])
+        else:
+            service_types = SubnetServiceType.get_objects(self.obj_context,
+                                                          subnet_id=self.id)
+
+        self.service_types = [service_type['service_type'] for
+                              service_type in service_types]
+        self.obj_reset_changes(['service_types'])
+
     def from_db_object(self, db_obj):
         super(Subnet, self).from_db_object(db_obj)
         self._load_shared(db_obj)
+        self._load_service_types(db_obj)
 
     @classmethod
     def modify_fields_from_db(cls, db_obj):
