@@ -22,9 +22,8 @@ import oslo_messaging
 from oslo_utils import excutils
 from osprofiler import profiler
 
-from neutron._i18n import _LE, _LI, _LW
+from neutron.agent.common import ovs_lib
 from neutron.common import utils as n_utils
-from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 
 LOG = logging.getLogger(__name__)
@@ -169,13 +168,13 @@ class OVSDVRNeutronAgent(object):
         try:
             self.get_dvr_mac_address_with_retry()
         except oslo_messaging.RemoteError as e:
-            LOG.error(_LE('L2 agent could not get DVR MAC address at '
-                          'startup due to RPC error.  It happens when the '
-                          'server does not support this RPC API.  Detailed '
-                          'message: %s'), e)
+            LOG.error('L2 agent could not get DVR MAC address at '
+                      'startup due to RPC error.  It happens when the '
+                      'server does not support this RPC API.  Detailed '
+                      'message: %s', e)
         except oslo_messaging.MessagingTimeout:
-            LOG.error(_LE('DVR: Failed to obtain a valid local '
-                          'DVR MAC address'))
+            LOG.error('DVR: Failed to obtain a valid local '
+                      'DVR MAC address')
 
         if not self.in_distributed_mode():
             sys.exit(1)
@@ -192,9 +191,9 @@ class OVSDVRNeutronAgent(object):
                 with excutils.save_and_reraise_exception() as ctx:
                     if retry_count > 0:
                         ctx.reraise = False
-                        LOG.warning(_LW('L2 agent could not get DVR MAC '
-                                        'address from server. Retrying. '
-                                        'Detailed message: %s'), e)
+                        LOG.warning('L2 agent could not get DVR MAC '
+                                    'address from server. Retrying. '
+                                    'Detailed message: %s', e)
             else:
                 LOG.debug("L2 Agent DVR: Received response for "
                           "get_dvr_mac_address_by_host() from "
@@ -205,11 +204,11 @@ class OVSDVRNeutronAgent(object):
     def setup_dvr_flows_on_integ_br(self):
         '''Setup up initial dvr flows into br-int'''
 
-        LOG.info(_LI("L2 Agent operating in DVR Mode with MAC %s"),
+        LOG.info("L2 Agent operating in DVR Mode with MAC %s",
                  self.dvr_mac_address)
         # Remove existing flows in integration bridge
         if self.conf.AGENT.drop_flows_on_start:
-            self.int_br.delete_flows()
+            self.int_br.uninstall_flows(cookie=ovs_lib.COOKIE_ANY)
 
         # Add a canary flow to int_br to track OVS restarts
         self.int_br.setup_canary_table()
@@ -220,10 +219,6 @@ class OVSDVRNeutronAgent(object):
         self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC_VLAN,
                                  priority=1)
 
-        # Insert 'normal' action as the default for Table LOCAL_SWITCHING
-        self.int_br.install_normal(table_id=constants.LOCAL_SWITCHING,
-                                   priority=1)
-
         for physical_network in self.bridge_mappings:
             self.int_br.install_drop(table_id=constants.LOCAL_SWITCHING,
                                      priority=2,
@@ -357,23 +352,16 @@ class OVSDVRNeutronAgent(object):
         # IP, directly use fixed_ips[0]
         fixed_ip = fixed_ips[0]
         subnet_uuid = fixed_ip['subnet_id']
-        csnat_ofport = constants.OFPORT_INVALID
-        ldm = None
         if subnet_uuid in self.local_dvr_map:
             ldm = self.local_dvr_map[subnet_uuid]
-            csnat_ofport = ldm.get_csnat_ofport()
-            if csnat_ofport == constants.OFPORT_INVALID:
-                LOG.error(_LE("DVR: Duplicate DVR router interface detected "
-                              "for subnet %s"), subnet_uuid)
-                return
         else:
             # set up LocalDVRSubnetMapping available for this subnet
             subnet_info = self.plugin_rpc.get_subnet_for_dvr(
                 self.context, subnet_uuid, fixed_ips=fixed_ips)
             if not subnet_info:
-                LOG.warning(_LW("DVR: Unable to retrieve subnet information "
-                                "for subnet_id %s. The subnet or the gateway "
-                                "may have already been deleted"), subnet_uuid)
+                LOG.warning("DVR: Unable to retrieve subnet information "
+                            "for subnet_id %s. The subnet or the gateway "
+                            "may have already been deleted", subnet_uuid)
                 return
             LOG.debug("get_subnet_for_dvr for subnet %(uuid)s "
                       "returned with %(info)s",
@@ -385,7 +373,7 @@ class OVSDVRNeutronAgent(object):
         ldm.set_dvr_owned(True)
 
         vlan_to_use = lvm.vlan
-        if lvm.network_type == p_const.TYPE_VLAN:
+        if lvm.network_type == n_const.TYPE_VLAN:
             vlan_to_use = lvm.segmentation_id
 
         subnet_info = ldm.get_subnet_info()
@@ -424,7 +412,7 @@ class OVSDVRNeutronAgent(object):
                 dst_mac=comp_ovsport.get_mac(),
                 dst_port=comp_ovsport.get_ofport())
 
-        if lvm.network_type == p_const.TYPE_VLAN:
+        if lvm.network_type == n_const.TYPE_VLAN:
             # TODO(vivek) remove the IPv6 related flows once SNAT is not
             # used for IPv6 DVR.
             br = self.phys_brs[lvm.physical_network]
@@ -485,7 +473,7 @@ class OVSDVRNeutronAgent(object):
                 ovsport.add_subnet(subnet_uuid)
                 self.local_ports[port.vif_id] = ovsport
             vlan_to_use = lvm.vlan
-            if lvm.network_type == p_const.TYPE_VLAN:
+            if lvm.network_type == n_const.TYPE_VLAN:
                 vlan_to_use = lvm.segmentation_id
             # create a rule for this vm port
             self.int_br.install_dvr_to_src_mac(
@@ -507,13 +495,13 @@ class OVSDVRNeutronAgent(object):
             subs = list(ovsport.get_subnets())
             if subs[0] == fixed_ip['subnet_id']:
                 return
-            LOG.error(_LE("Centralized-SNAT port %(port)s on subnet "
-                          "%(port_subnet)s already seen on a different "
-                          "subnet %(orig_subnet)s"), {
-                "port": port.vif_id,
-                "port_subnet": fixed_ip['subnet_id'],
-                "orig_subnet": subs[0],
-            })
+            LOG.error("Centralized-SNAT port %(port)s on subnet "
+                      "%(port_subnet)s already seen on a different "
+                      "subnet %(orig_subnet)s", {
+                          "port": port.vif_id,
+                          "port_subnet": fixed_ip['subnet_id'],
+                          "orig_subnet": subs[0],
+                      })
             return
         subnet_uuid = fixed_ip['subnet_id']
         ldm = None
@@ -524,9 +512,9 @@ class OVSDVRNeutronAgent(object):
             subnet_info = self.plugin_rpc.get_subnet_for_dvr(
                 self.context, subnet_uuid, fixed_ips=fixed_ips)
             if not subnet_info:
-                LOG.warning(_LW("DVR: Unable to retrieve subnet information "
-                                "for subnet_id %s. The subnet or the gateway "
-                                "may have already been deleted"), subnet_uuid)
+                LOG.warning("DVR: Unable to retrieve subnet information "
+                            "for subnet_id %s. The subnet or the gateway "
+                            "may have already been deleted", subnet_uuid)
                 return
             LOG.debug("get_subnet_for_dvr for subnet %(uuid)s "
                       "returned with %(info)s",
@@ -545,7 +533,7 @@ class OVSDVRNeutronAgent(object):
         ovsport.add_subnet(subnet_uuid)
         self.local_ports[port.vif_id] = ovsport
         vlan_to_use = lvm.vlan
-        if lvm.network_type == p_const.TYPE_VLAN:
+        if lvm.network_type == n_const.TYPE_VLAN:
             vlan_to_use = lvm.segmentation_id
         self.int_br.install_dvr_to_src_mac(
             network_type=lvm.network_type,
@@ -560,7 +548,7 @@ class OVSDVRNeutronAgent(object):
             return
 
         if local_vlan_map.network_type not in (constants.TUNNEL_NETWORK_TYPES
-                                               + [p_const.TYPE_VLAN]):
+                                               + [n_const.TYPE_VLAN]):
             LOG.debug("DVR: Port %s is with network_type %s not supported"
                       " for dvr plumbing", port.vif_id,
                       local_vlan_map.network_type)
@@ -568,8 +556,8 @@ class OVSDVRNeutronAgent(object):
 
         if (port.vif_id in self.local_ports and
                 self.local_ports[port.vif_id].ofport != port.ofport):
-            LOG.info(_LI("DVR: Port %(vif)s changed port number to "
-                         "%(ofport)s, rebinding."),
+            LOG.info("DVR: Port %(vif)s changed port number to "
+                     "%(ofport)s, rebinding.",
                      {'vif': port.vif_id, 'ofport': port.ofport})
             self.unbind_port_from_dvr(port, local_vlan_map)
         if device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
@@ -597,7 +585,7 @@ class OVSDVRNeutronAgent(object):
         network_type = lvm.network_type
         physical_network = lvm.physical_network
         vlan_to_use = lvm.vlan
-        if network_type == p_const.TYPE_VLAN:
+        if network_type == n_const.TYPE_VLAN:
             vlan_to_use = lvm.segmentation_id
         # ensure we process for all the subnets laid on this removed port
         for sub_uuid in subnet_set:
@@ -623,7 +611,7 @@ class OVSDVRNeutronAgent(object):
                 # this subnet from local_dvr_map, as no dvr (or) csnat
                 # ports available on this agent anymore
                 self.local_dvr_map.pop(sub_uuid, None)
-            if network_type == p_const.TYPE_VLAN:
+            if network_type == n_const.TYPE_VLAN:
                 br = self.phys_brs[physical_network]
             if network_type in constants.TUNNEL_NETWORK_TYPES:
                 br = self.tun_br
@@ -635,7 +623,7 @@ class OVSDVRNeutronAgent(object):
                     vlan_tag=lvm.vlan, gateway_mac=subnet_info['gateway_mac'])
             ovsport.remove_subnet(sub_uuid)
 
-        if lvm.network_type == p_const.TYPE_VLAN:
+        if lvm.network_type == n_const.TYPE_VLAN:
             br = self.phys_brs[physical_network]
         if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
             br = self.tun_br
@@ -657,7 +645,7 @@ class OVSDVRNeutronAgent(object):
             ldm = self.local_dvr_map[sub_uuid]
             ldm.remove_compute_ofport(port.vif_id)
             vlan_to_use = lvm.vlan
-            if lvm.network_type == p_const.TYPE_VLAN:
+            if lvm.network_type == n_const.TYPE_VLAN:
                 vlan_to_use = lvm.segmentation_id
             # first remove this vm port rule
             self.int_br.delete_dvr_to_src_mac(
@@ -678,7 +666,7 @@ class OVSDVRNeutronAgent(object):
         ldm = self.local_dvr_map[sub_uuid]
         ldm.set_csnat_ofport(constants.OFPORT_INVALID)
         vlan_to_use = lvm.vlan
-        if lvm.network_type == p_const.TYPE_VLAN:
+        if lvm.network_type == n_const.TYPE_VLAN:
             vlan_to_use = lvm.segmentation_id
         # then remove csnat port rule
         self.int_br.delete_dvr_to_src_mac(
