@@ -11,13 +11,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.extensions import portbindings
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
     constants as agent_consts)
 from neutron.services.trunk import constants as trunk_consts
@@ -68,8 +68,9 @@ def register():
     LOG.debug('Open vSwitch trunk driver registered')
 
 
-def vif_details_bridge_name_handler(resource, event, set_br_name, **kwargs):
+def vif_details_bridge_name_handler(resource, event, set_br_name,
+                                    payload=None):
     """If port is a trunk port, generate a bridge_name for its vif_details."""
-    port = kwargs['port']
+    port = payload.metadata['port']
     if 'trunk_details' in port:
         set_br_name(utils.gen_trunk_br_name(port['trunk_details']['trunk_id']))
