@@ -19,19 +19,19 @@ import random
 import eventlet
 import mock
 from neutron_lib import constants as n_const
+from neutron_lib.utils import net
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
-from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
 from neutron.agent.l2 import l2_agent_extensions_manager as ext_manager
 from neutron.agent.linux import interface
 from neutron.agent.linux import polling
 from neutron.common import utils
+from neutron.conf.agent import common as agent_config
 from neutron.conf import common as common_config
-from neutron.plugins.common import constants as p_const
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import config \
-    as ovs_config
+from neutron.conf.plugins.ml2.drivers import agent
+from neutron.conf.plugins.ml2.drivers import ovs_conf
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
     import br_int
@@ -70,9 +70,9 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
     def _get_config_opts(self):
         config = cfg.ConfigOpts()
         config.register_opts(common_config.core_opts)
-        config.register_opts(interface.OPTS)
-        config.register_opts(ovs_config.ovs_opts, "OVS")
-        config.register_opts(ovs_config.agent_opts, "AGENT")
+        agent.register_agent_opts(config)
+        ovs_conf.register_ovs_agent_opts(config)
+        agent_config.register_interface_opts(config)
         agent_config.register_interface_driver_opts_helper(config)
         agent_config.register_agent_state_opts_helper(config)
         ext_manager.register_opts(config)
@@ -101,19 +101,19 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
     def create_agent(self, create_tunnels=True, ancillary_bridge=None,
                      local_ip='192.168.10.1'):
         if create_tunnels:
-            tunnel_types = [p_const.TYPE_VXLAN]
+            tunnel_types = [n_const.TYPE_VXLAN]
         else:
             tunnel_types = None
         bridge_mappings = ['physnet:%s' % self.br_phys]
         self.config.set_override('tunnel_types', tunnel_types, "AGENT")
         self.config.set_override('polling_interval', 1, "AGENT")
-        self.config.set_override('prevent_arp_spoofing', False, "AGENT")
         self.config.set_override('local_ip', local_ip, "OVS")
         self.config.set_override('bridge_mappings', bridge_mappings, "OVS")
         # Physical bridges should be created prior to running
         self._bridge_classes()['br_phys'](self.br_phys).create()
+        ext_mgr = ext_manager.L2AgentExtensionsManager(self.config)
         agent = ovs_agent.OVSNeutronAgent(self._bridge_classes(),
-                                          self.config)
+                                          ext_mgr, self.config)
         self.addCleanup(self.ovs.delete_bridge, self.br_int)
         if tunnel_types:
             self.addCleanup(self.ovs.delete_bridge, self.br_tun)
@@ -171,7 +171,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
 
     def _create_test_port_dict(self):
         return {'id': uuidutils.generate_uuid(),
-                'mac_address': utils.get_random_mac(
+                'mac_address': net.get_random_mac(
                     'fa:16:3e:00:00:00'.split(':')),
                 'fixed_ips': [{
                     'ip_address': '10.%d.%d.%d' % (
@@ -214,8 +214,6 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                'segmentation_id': network.get('segmentation_id', 1),
                'fixed_ips': port['fixed_ips'],
                'device_owner': n_const.DEVICE_OWNER_COMPUTE_PREFIX,
-               'port_security_enabled': True,
-               'security_groups': ['default'],
                'admin_state_up': True}
         return dev
 
