@@ -85,3 +85,9 @@ class SanityTestCaseRoot(functional_base.BaseSudoTestCase):
 
     def test_keepalived_ipv6_support(self):
         checks.keepalived_ipv6_supported()
+
+    def test_bridge_firewalling_enabled(self):
+        checks.bridge_firewalling_enabled()
+
+    def test_ip_nonlocal_bind(self):
+        checks.ip_nonlocal_bind()
