@@ -16,11 +16,18 @@
 import os
 
 import mock
+from neutron_lib.api import converters
+from neutron_lib.callbacks import registry
+from neutron_lib import constants
+from neutron_lib import context
+from neutron_lib import exceptions as n_exc
+from neutron_lib import fixture
+from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_db import exception as db_exc
 from oslo_policy import policy as oslo_policy
 from oslo_utils import uuidutils
 import six
-from six import moves
 import six.moves.urllib.parse as urlparse
 import webob
 from webob import exc
@@ -28,19 +35,15 @@ import webtest
 
 from neutron.api import api_common
 from neutron.api import extensions
-from neutron.api.rpc.agentnotifiers import dhcp_rpc_agent_api
 from neutron.api.v2 import attributes
 from neutron.api.v2 import base as v2_base
 from neutron.api.v2 import router
-from neutron.common import exceptions as n_exc
-from neutron import context
-from neutron import manager
 from neutron import policy
 from neutron import quota
 from neutron.quota import resource_registry
 from neutron.tests import base
 from neutron.tests import fake_notifier
-from neutron.tests import tools
+from neutron.tests.unit import dummy_plugin
 from neutron.tests.unit import testlib_api
 
 
@@ -49,7 +52,8 @@ EXTDIR = os.path.join(base.ROOTDIR, 'unit/extensions')
 _uuid = uuidutils.generate_uuid
 
 
-def _get_path(resource, id=None, action=None, fmt=None):
+def _get_path(resource, id=None, action=None,
+              fmt=None, endpoint=None):
     path = '/%s' % resource
 
     if id is not None:
@@ -58,37 +62,15 @@ def _get_path(resource, id=None, action=None, fmt=None):
     if action is not None:
         path = path + '/%s' % action
 
+    if endpoint is not None:
+        path = path + '/%s' % endpoint
+
     if fmt is not None:
         path = path + '.%s' % fmt
 
     return path
 
 
-class ResourceIndexTestCase(base.BaseTestCase):
-    def test_index_json(self):
-        index = webtest.TestApp(router.Index({'foo': 'bar'}))
-        res = index.get('')
-
-        self.assertIn('resources', res.json)
-        self.assertEqual(len(res.json['resources']), 1)
-
-        resource = res.json['resources'][0]
-        self.assertIn('collection', resource)
-        self.assertEqual(resource['collection'], 'bar')
-
-        self.assertIn('name', resource)
-        self.assertEqual(resource['name'], 'foo')
-
-        self.assertIn('links', resource)
-        self.assertEqual(len(resource['links']), 1)
-
-        link = resource['links'][0]
-        self.assertIn('href', link)
-        self.assertEqual(link['href'], 'http://localhost/bar')
-        self.assertIn('rel', link)
-        self.assertEqual(link['rel'], 'self')
-
-
 class APIv2TestBase(base.BaseTestCase):
     def setUp(self):
         super(APIv2TestBase, self).setUp()
@@ -99,9 +81,7 @@ class APIv2TestBase(base.BaseTestCase):
         # Create the default configurations
         self.config_parse()
         # Update the plugin
-        self.setup_coreplugin(plugin)
-        cfg.CONF.set_override('allow_pagination', True)
-        cfg.CONF.set_override('allow_sorting', True)
+        self.setup_coreplugin(plugin, load_plugins=False)
         self._plugin_patcher = mock.patch(plugin, autospec=True)
         self.plugin = self._plugin_patcher.start()
         instance = self.plugin.return_value
@@ -135,10 +115,18 @@ def _list_cmp(l1, l2):
 
 
 class APIv2TestCase(APIv2TestBase):
+
+    @staticmethod
+    def _get_policy_attrs(attr_info):
+        policy_attrs = {name for (name, info) in attr_info.items()
+                        if info.get('required_by_policy')}
+        if 'tenant_id' in policy_attrs:
+            policy_attrs.add('project_id')
+        return sorted(policy_attrs)
+
     def _do_field_list(self, resource, base_fields):
         attr_info = attributes.RESOURCE_ATTRIBUTE_MAP[resource]
-        policy_attrs = [name for (name, info) in attr_info.items()
-                        if info.get('required_by_policy')]
+        policy_attrs = self._get_policy_attrs(attr_info)
         for name, info in attr_info.items():
             if info.get('primary_key'):
                 policy_attrs.append(name)
@@ -146,7 +134,8 @@ class APIv2TestCase(APIv2TestBase):
         fields.extend(policy_attrs)
         return fields
 
-    def _get_collection_kwargs(self, skipargs=[], **kwargs):
+    def _get_collection_kwargs(self, skipargs=None, **kwargs):
+        skipargs = skipargs or []
         args_list = ['filters', 'fields', 'sorts', 'limit', 'marker',
                      'page_reverse']
         args_dict = dict(
@@ -167,7 +156,7 @@ class APIv2TestCase(APIv2TestBase):
         instance = self.plugin.return_value
         instance.get_networks.return_value = []
 
-        fields = self._do_field_list('networks', ['foo', 'bar'])
+        fields = self._do_field_list('networks', ['bar', 'foo'])
         self.api.get(_get_path('networks'), {'fields': ['foo', 'bar']})
         kwargs = self._get_collection_kwargs(fields=fields)
         instance.get_networks.assert_called_once_with(mock.ANY, **kwargs)
@@ -325,7 +314,7 @@ class APIv2TestCase(APIv2TestBase):
 
         res = self.api.get(_get_path('networks'), {'limit': -1},
                            expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_limit_with_non_integer(self):
         instance = self.plugin.return_value
@@ -333,7 +322,8 @@ class APIv2TestCase(APIv2TestBase):
 
         res = self.api.get(_get_path('networks'),
                            {'limit': 'abc'}, expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
+        self.assertIn('abc', res)
 
     def test_limit_with_infinite_pagination_max_limit(self):
         instance = self.plugin.return_value
@@ -435,7 +425,7 @@ class APIv2TestCase(APIv2TestBase):
 
         res = self.api.get(_get_path('networks'), {'sort_key': ['name']},
                            expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_sort_with_invalid_attribute(self):
         instance = self.plugin.return_value
@@ -445,7 +435,7 @@ class APIv2TestCase(APIv2TestBase):
                            {'sort_key': 'abc',
                             'sort_dir': 'asc'},
                            expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_sort_with_invalid_dirs(self):
         instance = self.plugin.return_value
@@ -455,7 +445,7 @@ class APIv2TestCase(APIv2TestBase):
                            {'sort_key': 'name',
                             'sort_dir': 'abc'},
                            expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_emulated_sort(self):
         instance = self.plugin.return_value
@@ -485,6 +475,7 @@ class APIv2TestCase(APIv2TestBase):
                                            'id',
                                            'subnets',
                                            'shared',
+                                           'project_id',
                                            'tenant_id']))
         instance.get_networks.assert_called_once_with(mock.ANY, **kwargs)
 
@@ -506,19 +497,6 @@ class APIv2TestCase(APIv2TestBase):
         instance._NeutronPluginBaseV2__native_sorting_support = False
         self.assertRaises(n_exc.Invalid, router.APIRouter)
 
-    def test_native_pagination_without_allow_sorting(self):
-        cfg.CONF.set_override('allow_sorting', False)
-        instance = self.plugin.return_value
-        instance.get_networks.return_value = []
-        api = webtest.TestApp(router.APIRouter())
-        api.get(_get_path('networks'),
-                {'sort_key': ['name', 'admin_state_up'],
-                 'sort_dir': ['desc', 'asc']})
-        kwargs = self._get_collection_kwargs(sorts=[('name', False),
-                                                    ('admin_state_up', True),
-                                                    ('id', True)])
-        instance.get_networks.assert_called_once_with(mock.ANY, **kwargs)
-
 
 # Note: since all resources use the same controller and validation
 # logic, we actually get really good coverage from testing just networks.
@@ -545,15 +523,15 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         self.assertIn('networks', res)
         if not req_tenant_id or req_tenant_id == real_tenant_id:
             # expect full list returned
-            self.assertEqual(len(res['networks']), 1)
+            self.assertEqual(1, len(res['networks']))
             output_dict = res['networks'][0]
             input_dict['shared'] = False
             self.assertEqual(len(input_dict), len(output_dict))
-            for k, v in six.iteritems(input_dict):
+            for k, v in input_dict.items():
                 self.assertEqual(v, output_dict[k])
         else:
             # expect no results
-            self.assertEqual(len(res['networks']), 0)
+            self.assertEqual(0, len(res['networks']))
 
     def test_list_noauth(self):
         self._test_list(None, _uuid())
@@ -593,7 +571,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.get(_get_path('networks'),
                            params=params).json
 
-        self.assertEqual(len(res['networks']), 2)
+        self.assertEqual(2, len(res['networks']))
         self.assertEqual(sorted([id1, id2]),
                          sorted([res['networks'][0]['id'],
                                 res['networks'][1]['id']]))
@@ -606,19 +584,19 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                 next_links.append(r)
             if r['rel'] == 'previous':
                 previous_links.append(r)
-        self.assertEqual(len(next_links), 1)
-        self.assertEqual(len(previous_links), 1)
+        self.assertEqual(1, len(next_links))
+        self.assertEqual(1, len(previous_links))
 
         url = urlparse.urlparse(next_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         params['marker'] = [id2]
-        self.assertEqual(urlparse.parse_qs(url.query), params)
+        self.assertEqual(params, urlparse.parse_qs(url.query))
 
         url = urlparse.urlparse(previous_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         params['marker'] = [id1]
         params['page_reverse'] = ['True']
-        self.assertEqual(urlparse.parse_qs(url.query), params)
+        self.assertEqual(params, urlparse.parse_qs(url.query))
 
     def test_list_pagination_with_last_page(self):
         id = str(_uuid())
@@ -637,7 +615,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.get(_get_path('networks'),
                            params=params).json
 
-        self.assertEqual(len(res['networks']), 1)
+        self.assertEqual(1, len(res['networks']))
         self.assertEqual(id, res['networks'][0]['id'])
 
         self.assertIn('networks_links', res)
@@ -646,14 +624,14 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
             self.assertNotEqual(r['rel'], 'next')
             if r['rel'] == 'previous':
                 previous_links.append(r)
-        self.assertEqual(len(previous_links), 1)
+        self.assertEqual(1, len(previous_links))
 
         url = urlparse.urlparse(previous_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         expect_params = params.copy()
         expect_params['marker'] = [id]
         expect_params['page_reverse'] = ['True']
-        self.assertEqual(urlparse.parse_qs(url.query), expect_params)
+        self.assertEqual(expect_params, urlparse.parse_qs(url.query))
 
     def test_list_pagination_with_empty_page(self):
         return_value = []
@@ -664,7 +642,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.get(_get_path('networks'),
                            params=params).json
 
-        self.assertEqual(res['networks'], [])
+        self.assertEqual([], res['networks'])
 
         previous_links = []
         if 'networks_links' in res:
@@ -672,14 +650,14 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                 self.assertNotEqual(r['rel'], 'next')
                 if r['rel'] == 'previous':
                     previous_links.append(r)
-        self.assertEqual(len(previous_links), 1)
+        self.assertEqual(1, len(previous_links))
 
         url = urlparse.urlparse(previous_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         expect_params = params.copy()
         del expect_params['marker']
         expect_params['page_reverse'] = ['True']
-        self.assertEqual(urlparse.parse_qs(url.query), expect_params)
+        self.assertEqual(expect_params, urlparse.parse_qs(url.query))
 
     def test_list_pagination_reverse_with_last_page(self):
         id = str(_uuid())
@@ -708,15 +686,15 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
             self.assertNotEqual(r['rel'], 'previous')
             if r['rel'] == 'next':
                 next_links.append(r)
-        self.assertEqual(len(next_links), 1)
+        self.assertEqual(1, len(next_links))
 
         url = urlparse.urlparse(next_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         expected_params = params.copy()
         del expected_params['page_reverse']
         expected_params['marker'] = [id]
-        self.assertEqual(urlparse.parse_qs(url.query),
-                         expected_params)
+        self.assertEqual(expected_params,
+                         urlparse.parse_qs(url.query))
 
     def test_list_pagination_reverse_with_empty_page(self):
         return_value = []
@@ -727,7 +705,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                   'page_reverse': ['True']}
         res = self.api.get(_get_path('networks'),
                            params=params).json
-        self.assertEqual(res['networks'], [])
+        self.assertEqual([], res['networks'])
 
         next_links = []
         if 'networks_links' in res:
@@ -735,14 +713,14 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                 self.assertNotEqual(r['rel'], 'previous')
                 if r['rel'] == 'next':
                     next_links.append(r)
-        self.assertEqual(len(next_links), 1)
+        self.assertEqual(1, len(next_links))
 
         url = urlparse.urlparse(next_links[0]['href'])
         self.assertEqual(url.path, _get_path('networks'))
         expect_params = params.copy()
         del expect_params['marker']
         del expect_params['page_reverse']
-        self.assertEqual(urlparse.parse_qs(url.query), expect_params)
+        self.assertEqual(expect_params, urlparse.parse_qs(url.query))
 
     def test_create(self):
         net_id = _uuid()
@@ -759,16 +737,20 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.post(_get_path('networks', fmt=self.fmt),
                             self.serialize(data),
                             content_type='application/' + self.fmt)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
         res = self.deserialize(res)
         self.assertIn('network', res)
         net = res['network']
-        self.assertEqual(net['id'], net_id)
-        self.assertEqual(net['status'], "ACTIVE")
+        self.assertEqual(net_id, net['id'])
+        self.assertEqual("ACTIVE", net['status'])
 
     def test_create_use_defaults(self):
         net_id = _uuid()
-        initial_input = {'network': {'name': 'net1', 'tenant_id': _uuid()}}
+        tenant_id = _uuid()
+
+        initial_input = {'network': {'name': 'net1',
+                                     'tenant_id': tenant_id,
+                                     'project_id': tenant_id}}
         full_input = {'network': {'admin_state_up': True,
                                   'shared': False}}
         full_input['network'].update(initial_input['network'])
@@ -785,13 +767,13 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                             content_type='application/' + self.fmt)
         instance.create_network.assert_called_with(mock.ANY,
                                                    network=full_input)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
         res = self.deserialize(res)
         self.assertIn('network', res)
         net = res['network']
-        self.assertEqual(net['id'], net_id)
-        self.assertEqual(net['admin_state_up'], True)
-        self.assertEqual(net['status'], "ACTIVE")
+        self.assertEqual(net_id, net['id'])
+        self.assertTrue(net['admin_state_up'])
+        self.assertEqual("ACTIVE", net['status'])
 
     def test_create_no_keystone_env(self):
         data = {'name': 'net1'}
@@ -804,7 +786,8 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         # tenant_id should be fetched from env
         initial_input = {'network': {'name': 'net1'}}
         full_input = {'network': {'admin_state_up': True,
-                      'shared': False, 'tenant_id': tenant_id}}
+                      'shared': False, 'tenant_id': tenant_id,
+                      'project_id': tenant_id}}
         full_input['network'].update(initial_input['network'])
 
         return_value = {'id': net_id, 'status': "ACTIVE"}
@@ -821,7 +804,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
 
         instance.create_network.assert_called_with(mock.ANY,
                                                    network=full_input)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
 
     def test_create_bad_keystone_tenant(self):
         tenant_id = _uuid()
@@ -863,7 +846,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                             self.serialize(data),
                             content_type='application/' + self.fmt,
                             expect_errors=True)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_create_bulk(self):
         data = {'networks': [{'name': 'net1',
@@ -884,14 +867,14 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.post(_get_path('networks', fmt=self.fmt),
                             self.serialize(data),
                             content_type='application/' + self.fmt)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
 
     def _test_create_failure_bad_request(self, resource, data, **kwargs):
         res = self.api.post(_get_path(resource, fmt=self.fmt),
                             self.serialize(data),
                             content_type='application/' + self.fmt,
                             expect_errors=True, **kwargs)
-        self.assertEqual(res.status_int, exc.HTTPBadRequest.code)
+        self.assertEqual(exc.HTTPBadRequest.code, res.status_int)
 
     def test_create_bulk_networks_none(self):
         self._test_create_failure_bad_request('networks', {'networks': None})
@@ -915,11 +898,12 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         device_id = _uuid()
         initial_input = {'port': {'name': '', 'network_id': net_id,
                                   'tenant_id': tenant_id,
+                                  'project_id': tenant_id,
                                   'device_id': device_id,
                                   'admin_state_up': True}}
         full_input = {'port': {'admin_state_up': True,
-                               'mac_address': attributes.ATTR_NOT_SPECIFIED,
-                               'fixed_ips': attributes.ATTR_NOT_SPECIFIED,
+                               'mac_address': constants.ATTR_NOT_SPECIFIED,
+                               'fixed_ips': constants.ATTR_NOT_SPECIFIED,
                                'device_owner': ''}}
         full_input['port'].update(initial_input['port'])
         return_value = {'id': _uuid(), 'status': 'ACTIVE',
@@ -939,12 +923,12 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                             self.serialize(initial_input),
                             content_type='application/' + self.fmt)
         instance.create_port.assert_called_with(mock.ANY, port=full_input)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
         res = self.deserialize(res)
         self.assertIn('port', res)
         port = res['port']
-        self.assertEqual(port['network_id'], net_id)
-        self.assertEqual(port['mac_address'], 'ca:fe:de:ad:be:ef')
+        self.assertEqual(net_id, port['network_id'])
+        self.assertEqual('ca:fe:de:ad:be:ef', port['mac_address'])
 
     def test_create_return_extra_attr(self):
         net_id = _uuid()
@@ -961,12 +945,12 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         res = self.api.post(_get_path('networks', fmt=self.fmt),
                             self.serialize(data),
                             content_type='application/' + self.fmt)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
         res = self.deserialize(res)
         self.assertIn('network', res)
         net = res['network']
-        self.assertEqual(net['id'], net_id)
-        self.assertEqual(net['status'], "ACTIVE")
+        self.assertEqual(net_id, net['id'])
+        self.assertEqual("ACTIVE", net['status'])
         self.assertNotIn('v2attrs:something', net)
 
     def test_fields(self):
@@ -995,7 +979,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                                         fmt=self.fmt),
                               extra_environ=env,
                               expect_errors=expect_errors)
-        self.assertEqual(res.status_int, expected_code)
+        self.assertEqual(expected_code, res.status_int)
 
     def test_delete_noauth(self):
         self._test_delete(None, _uuid(), exc.HTTPNoContent.code)
@@ -1028,7 +1012,7 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                                      fmt=self.fmt),
                            extra_environ=env,
                            expect_errors=expect_errors)
-        self.assertEqual(res.status_int, expected_code)
+        self.assertEqual(expected_code, res.status_int)
         return res
 
     def test_get_noauth(self):
@@ -1096,13 +1080,18 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
         self._test_update(tenant_id + "bad", tenant_id,
                           exc.HTTPNotFound.code, expect_errors=True)
 
+    def test_update_keystone_no_tenant(self):
+        tenant_id = _uuid()
+        self._test_update(tenant_id, None,
+                          exc.HTTPNotFound.code, expect_errors=True)
+
     def test_update_readonly_field(self):
         data = {'network': {'status': "NANANA"}}
         res = self.api.put(_get_path('networks', id=_uuid()),
                            self.serialize(data),
                            content_type='application/' + self.fmt,
                            expect_errors=True)
-        self.assertEqual(res.status_int, 400)
+        self.assertEqual(400, res.status_int)
 
     def test_invalid_attribute_field(self):
         data = {'network': {'invalid_key1': "foo1", 'invalid_key2': "foo2"}}
@@ -1110,20 +1099,34 @@ class JSONV2TestCase(APIv2TestBase, testlib_api.WebTestCase):
                            self.serialize(data),
                            content_type='application/' + self.fmt,
                            expect_errors=True)
-        self.assertEqual(res.status_int, 400)
+        self.assertEqual(400, res.status_int)
+
+    def test_retry_on_index(self):
+        instance = self.plugin.return_value
+        instance.get_networks.side_effect = [db_exc.RetryRequest(None), []]
+        api = webtest.TestApp(router.APIRouter())
+        api.get(_get_path('networks', fmt=self.fmt))
+        self.assertTrue(instance.get_networks.called)
+
+    def test_retry_on_show(self):
+        instance = self.plugin.return_value
+        instance.get_network.side_effect = [db_exc.RetryRequest(None), {}]
+        api = webtest.TestApp(router.APIRouter())
+        api.get(_get_path('networks', _uuid(), fmt=self.fmt))
+        self.assertTrue(instance.get_network.called)
 
 
 class SubresourceTest(base.BaseTestCase):
     def setUp(self):
         super(SubresourceTest, self).setUp()
-
+        raise self.skipException('this class will be deleted')
         plugin = 'neutron.tests.unit.api.v2.test_base.TestSubresourcePlugin'
         extensions.PluginAwareExtensionManager._instance = None
 
-        self.useFixture(tools.AttributeMapMemento())
+        self.useFixture(fixture.APIDefinitionFixture())
 
         self.config_parse()
-        self.setup_coreplugin(plugin)
+        self.setup_coreplugin(plugin, load_plugins=False)
 
         self._plugin_patcher = mock.patch(plugin, autospec=True)
         self.plugin = self._plugin_patcher.start()
@@ -1132,7 +1135,7 @@ class SubresourceTest(base.BaseTestCase):
 
         SUB_RESOURCES = {}
         RESOURCE_ATTRIBUTE_MAP = {}
-        SUB_RESOURCES['dummy'] = {
+        SUB_RESOURCES[dummy_plugin.RESOURCE_NAME] = {
             'collection_name': 'dummies',
             'parent': {'collection_name': 'networks',
                        'member_name': 'network'}
@@ -1146,12 +1149,13 @@ class SubresourceTest(base.BaseTestCase):
                           'required_by_policy': True,
                           'is_visible': True}
         }
-        collection_name = SUB_RESOURCES['dummy'].get('collection_name')
-        resource_name = 'dummy'
-        parent = SUB_RESOURCES['dummy'].get('parent')
+        collection_name = SUB_RESOURCES[
+            dummy_plugin.RESOURCE_NAME].get('collection_name')
+        resource_name = dummy_plugin.RESOURCE_NAME
+        parent = SUB_RESOURCES[dummy_plugin.RESOURCE_NAME].get('parent')
         params = RESOURCE_ATTRIBUTE_MAP['dummies']
         member_actions = {'mactions': 'GET'}
-        _plugin = manager.NeutronManager.get_plugin()
+        _plugin = directory.get_plugin()
         controller = v2_base.create_resource(collection_name, resource_name,
                                              _plugin, params,
                                              member_actions=member_actions,
@@ -1172,9 +1176,6 @@ class SubresourceTest(base.BaseTestCase):
                          member=member_actions)
         self.api = webtest.TestApp(api)
 
-    def tearDown(self):
-        super(SubresourceTest, self).tearDown()
-
     def test_index_sub_resource(self):
         instance = self.plugin.return_value
 
@@ -1196,8 +1197,14 @@ class SubresourceTest(base.BaseTestCase):
 
     def test_create_sub_resource(self):
         instance = self.plugin.return_value
+        tenant_id = _uuid()
 
-        body = {'dummy': {'foo': 'bar', 'tenant_id': _uuid()}}
+        body = {
+            dummy_plugin.RESOURCE_NAME: {
+                'foo': 'bar', 'tenant_id': tenant_id,
+                'project_id': tenant_id
+            }
+        }
         self.api.post_json('/networks/id1/dummies', body)
         instance.create_network_dummy.assert_called_once_with(mock.ANY,
                                                               network_id='id1',
@@ -1207,7 +1214,7 @@ class SubresourceTest(base.BaseTestCase):
         instance = self.plugin.return_value
 
         dummy_id = _uuid()
-        body = {'dummy': {'foo': 'bar'}}
+        body = {dummy_plugin.RESOURCE_NAME: {'foo': 'bar'}}
         self.api.put_json('/networks/id1' + _get_path('dummies', id=dummy_id),
                           body)
         instance.update_network_dummy.assert_called_once_with(mock.ANY,
@@ -1219,7 +1226,7 @@ class SubresourceTest(base.BaseTestCase):
         instance = self.plugin.return_value
 
         dummy_id = _uuid()
-        body = {'dummy': {}}
+        body = {dummy_plugin.RESOURCE_NAME: {}}
         self.api.put_json('/networks/id1' + _get_path('dummies', id=dummy_id),
                           body)
         instance.update_network_dummy.assert_called_once_with(mock.ANY,
@@ -1307,13 +1314,18 @@ class NotificationTest(APIv2TestBase):
 
         expected_events = ('.'.join([resource, opname, "start"]),
                            '.'.join([resource, opname, "end"]))
-        self.assertEqual(len(fake_notifier.NOTIFICATIONS),
-                         len(expected_events))
+        self.assertEqual(len(expected_events),
+                         len(fake_notifier.NOTIFICATIONS))
         for msg, event in zip(fake_notifier.NOTIFICATIONS, expected_events):
             self.assertEqual('INFO', msg['priority'])
             self.assertEqual(event, msg['event_type'])
+            if opname == 'delete' and event == 'network.delete.end':
+                self.assertIn('payload', msg)
+                resource = msg['payload']
+                self.assertIn('network_id', resource)
+                self.assertIn('network', resource)
 
-        self.assertEqual(res.status_int, expected_code)
+        self.assertEqual(expected_code, res.status_int)
 
     def test_network_create_notifer(self):
         self._resource_op_notifier('create', 'network')
@@ -1325,20 +1337,19 @@ class NotificationTest(APIv2TestBase):
         self._resource_op_notifier('update', 'network')
 
 
-class DHCPNotificationTest(APIv2TestBase):
+class RegistryNotificationTest(APIv2TestBase):
 
     def setUp(self):
         # This test does not have database support so tracking cannot be used
         cfg.CONF.set_override('track_quota_usage', False, group='QUOTAS')
-        super(DHCPNotificationTest, self).setUp()
+        super(RegistryNotificationTest, self).setUp()
 
-    def _test_dhcp_notifier(self, opname, resource, initial_input=None):
+    def _test_registry_notify(self, opname, resource, initial_input=None):
         instance = self.plugin.return_value
         instance.get_networks.return_value = initial_input
         instance.get_networks_count.return_value = 0
         expected_code = exc.HTTPCreated.code
-        with mock.patch.object(dhcp_rpc_agent_api.DhcpAgentNotifyAPI,
-                               'notify') as dhcp_notifier:
+        with mock.patch.object(registry, 'publish') as notify:
             if opname == 'create':
                 res = self.api.post_json(
                     _get_path('networks'),
@@ -1351,35 +1362,27 @@ class DHCPNotificationTest(APIv2TestBase):
             if opname == 'delete':
                 res = self.api.delete(_get_path('networks', id=_uuid()))
                 expected_code = exc.HTTPNoContent.code
-            expected_item = mock.call(mock.ANY, mock.ANY,
-                                      resource + "." + opname + ".end")
-            if initial_input and resource not in initial_input:
-                resource += 's'
-            num = len(initial_input[resource]) if initial_input and isinstance(
-                initial_input[resource], list) else 1
-            expected = [expected_item for x in moves.range(num)]
-            self.assertEqual(expected, dhcp_notifier.call_args_list)
-            self.assertEqual(num, dhcp_notifier.call_count)
+            self.assertTrue(notify.called)
         self.assertEqual(expected_code, res.status_int)
 
-    def test_network_create_dhcp_notifer(self):
+    def test_network_create_registry_notify(self):
         input = {'network': {'name': 'net',
                              'tenant_id': _uuid()}}
-        self._test_dhcp_notifier('create', 'network', input)
+        self._test_registry_notify('create', 'network', input)
 
-    def test_network_delete_dhcp_notifer(self):
-        self._test_dhcp_notifier('delete', 'network')
+    def test_network_delete_registry_notify(self):
+        self._test_registry_notify('delete', 'network')
 
-    def test_network_update_dhcp_notifer(self):
+    def test_network_update_registry_notify(self):
         input = {'network': {'name': 'net'}}
-        self._test_dhcp_notifier('update', 'network', input)
+        self._test_registry_notify('update', 'network', input)
 
-    def test_networks_create_bulk_dhcp_notifer(self):
+    def test_networks_create_bulk_registry_notify(self):
         input = {'networks': [{'name': 'net1',
                                'tenant_id': _uuid()},
                               {'name': 'net2',
                                'tenant_id': _uuid()}]}
-        self._test_dhcp_notifier('create', 'network', input)
+        self._test_registry_notify('create', 'network', input)
 
 
 class QuotaTest(APIv2TestBase):
@@ -1390,7 +1393,7 @@ class QuotaTest(APIv2TestBase):
         super(QuotaTest, self).setUp()
         # Use mock to let the API use a different QuotaEngine instance for
         # unit test in this class. This will ensure resource are registered
-        # again and instanciated with neutron.quota.resource.CountableResource
+        # again and instantiated with neutron.quota.resource.CountableResource
         replacement_registry = resource_registry.ResourceRegistry()
         registry_patcher = mock.patch('neutron.quota.resource_registry.'
                                       'ResourceRegistry.get_instance')
@@ -1439,7 +1442,7 @@ class QuotaTest(APIv2TestBase):
         instance.get_networks_count.return_value = 3
         res = self.api.post_json(
             _get_path('networks'), initial_input)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
 
 
 class ExtensionTestCase(base.BaseTestCase):
@@ -1451,21 +1454,20 @@ class ExtensionTestCase(base.BaseTestCase):
         # Ensure existing ExtensionManager is not used
         extensions.PluginAwareExtensionManager._instance = None
 
-        self.useFixture(tools.AttributeMapMemento())
+        self.useFixture(fixture.APIDefinitionFixture())
 
         # Create the default configurations
         self.config_parse()
 
         # Update the plugin and extensions path
-        self.setup_coreplugin(plugin)
+        self.setup_coreplugin(plugin, load_plugins=False)
         cfg.CONF.set_override('api_extensions_path', EXTDIR)
 
         self._plugin_patcher = mock.patch(plugin, autospec=True)
         self.plugin = self._plugin_patcher.start()
 
         # Instantiate mock plugin and enable the V2attributes extension
-        manager.NeutronManager.get_plugin().supported_extension_aliases = (
-            ["v2attrs"])
+        self.plugin.return_value.supported_extension_aliases = ["v2attrs"]
 
         api = router.APIRouter()
         self.api = webtest.TestApp(api)
@@ -1474,14 +1476,11 @@ class ExtensionTestCase(base.BaseTestCase):
         cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
                               group='QUOTAS')
 
-    def tearDown(self):
-        super(ExtensionTestCase, self).tearDown()
-        self.api = None
-        self.plugin = None
-
     def test_extended_create(self):
         net_id = _uuid()
-        initial_input = {'network': {'name': 'net1', 'tenant_id': _uuid(),
+        tenant_id = _uuid()
+        initial_input = {'network': {'name': 'net1', 'tenant_id': tenant_id,
+                                     'project_id': tenant_id,
                                      'v2attrs:something_else': "abc"}}
         data = {'network': {'admin_state_up': True, 'shared': False}}
         data['network'].update(initial_input['network'])
@@ -1499,12 +1498,12 @@ class ExtensionTestCase(base.BaseTestCase):
 
         instance.create_network.assert_called_with(mock.ANY,
                                                    network=data)
-        self.assertEqual(res.status_int, exc.HTTPCreated.code)
+        self.assertEqual(exc.HTTPCreated.code, res.status_int)
         self.assertIn('network', res.json)
         net = res.json['network']
-        self.assertEqual(net['id'], net_id)
-        self.assertEqual(net['status'], "ACTIVE")
-        self.assertEqual(net['v2attrs:something'], "123")
+        self.assertEqual(net_id, net['id'])
+        self.assertEqual("ACTIVE", net['status'])
+        self.assertEqual("123", net['v2attrs:something'])
         self.assertNotIn('v2attrs:something_else', net)
 
 
@@ -1536,7 +1535,7 @@ class ListArgsTestCase(base.BaseTestCase):
         request = webob.Request.blank(path)
         expect_val = ['2', '4']
         actual_val = api_common.list_args(request, 'fields')
-        self.assertEqual(sorted(actual_val), expect_val)
+        self.assertEqual(expect_val, sorted(actual_val))
 
     def test_list_args_with_empty(self):
         path = '/?foo=4&bar=3&baz=2&qux=1'
@@ -1561,7 +1560,7 @@ class FiltersTestCase(base.BaseTestCase):
         request = webob.Request.blank(path)
         expect_val = {'foo': ['4'], 'bar': ['3'], 'baz': ['2'], 'qux': ['1']}
         actual_val = api_common.get_filters(request, {})
-        self.assertEqual(actual_val, expect_val)
+        self.assertEqual(expect_val, actual_val)
 
     def test_attr_info_without_conversion(self):
         path = '/?foo=4&bar=3&baz=2&qux=1'
@@ -1569,14 +1568,14 @@ class FiltersTestCase(base.BaseTestCase):
         attr_info = {'foo': {'key': 'val'}}
         expect_val = {'foo': ['4'], 'bar': ['3'], 'baz': ['2'], 'qux': ['1']}
         actual_val = api_common.get_filters(request, attr_info)
-        self.assertEqual(actual_val, expect_val)
+        self.assertEqual(expect_val, actual_val)
 
     def test_attr_info_with_convert_list_to(self):
         path = '/?foo=key=4&bar=3&foo=key=2&qux=1'
         request = webob.Request.blank(path)
         attr_info = {
             'foo': {
-                'convert_list_to': attributes.convert_kvp_list_to_dict,
+                'convert_list_to': converters.convert_kvp_list_to_dict,
             }
         }
         expect_val = {'foo': {'key': ['2', '4']}, 'bar': ['3'], 'qux': ['1']}
@@ -1586,10 +1585,15 @@ class FiltersTestCase(base.BaseTestCase):
     def test_attr_info_with_convert_to(self):
         path = '/?foo=4&bar=3&baz=2&qux=1'
         request = webob.Request.blank(path)
-        attr_info = {'foo': {'convert_to': attributes.convert_to_int}}
+        attr_info = {'foo': {'convert_to': converters.convert_to_int}}
         expect_val = {'foo': [4], 'bar': ['3'], 'baz': ['2'], 'qux': ['1']}
         actual_val = api_common.get_filters(request, attr_info)
-        self.assertEqual(actual_val, expect_val)
+        self.assertEqual(expect_val, actual_val)
+
+    def test_attr_info_with_base_db_attributes(self):
+        path = '/?__contains__=1&__class__=2'
+        request = webob.Request.blank(path)
+        self.assertEqual({}, api_common.get_filters(request, {}))
 
 
 class CreateResourceTestCase(base.BaseTestCase):
