@@ -14,16 +14,21 @@
 # limitations under the License.
 
 import mock
+import netaddr
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib import context
 from neutron_lib import exceptions as n_exc
+from neutron_lib.exceptions import l3 as l3_exc
+from neutron_lib.plugins import directory
+from oslo_utils import uuidutils
 import testtools
 
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
 from neutron.db import l3_db
-from neutron.extensions import l3
-from neutron import manager
+from neutron.db.models import l3 as l3_models
+from neutron.objects import router as l3_obj
 from neutron.tests import base
 
 
@@ -37,6 +42,16 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
         filtered = l3_db.L3_NAT_dbonly_mixin._each_port_having_fixed_ips(None)
         self.assertEqual([], list(filtered))
 
+    def test__new__passes_args(self):
+        class T(l3_db.L3_NAT_db_mixin):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        t = T(1, 2, a=3)
+        self.assertEqual((1, 2), t.args)
+        self.assertEqual({'a': 3}, t.kwargs)
+
     def test__each_port_having_fixed_ips(self):
         """Basic test that ports without fixed ips are filtered out"""
         ports = [{'id': 'a', 'fixed_ips': [mock.sentinel.fixedip]},
@@ -48,7 +63,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
     def test__get_subnets_by_network_no_query(self):
         """Basic test that no query is performed if no Ports are passed"""
         context = mock.Mock()
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as get_p:
+        with mock.patch.object(directory, 'get_plugin') as get_p:
             self.db._get_subnets_by_network_list(context, [])
         self.assertFalse(context.session.query.called)
         self.assertFalse(get_p.called)
@@ -60,7 +75,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
         query.__iter__.return_value = [(mock.sentinel.subnet_db,
                                         mock.sentinel.address_scope_id)]
 
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as get_p:
+        with mock.patch.object(directory, 'get_plugin') as get_p:
             get_p()._make_subnet_dict.return_value = {
                 'network_id': mock.sentinel.network_id}
             subnets = self.db._get_subnets_by_network_list(
@@ -73,7 +88,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
     def test__populate_ports_for_subnets_none(self):
         """Basic test that the method runs correctly with no ports"""
         ports = []
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as get_p:
+        with mock.patch.object(directory, 'get_plugin') as get_p:
             get_p().get_networks.return_value = []
             self.db._populate_mtu_and_subnets_for_ports(mock.sentinel.context,
                                                         ports)
@@ -95,7 +110,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
         ports = [{'network_id': 'net_id',
                   'id': 'port_id',
                   'fixed_ips': [{'subnet_id': mock.sentinel.subnet_id}]}]
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as get_p:
+        with mock.patch.object(directory, 'get_plugin') as get_p:
             get_p().get_networks.return_value = [{'id': 'net_id', 'mtu': 1446}]
             self.db._populate_mtu_and_subnets_for_ports(mock.sentinel.context,
                                                         ports)
@@ -130,6 +145,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
             'id': mock.sentinel.fip_ip}, result)
 
     def test__unique_floatingip_iterator(self):
+        context = mock.MagicMock()
         query = mock.MagicMock()
         query.order_by().__iter__.return_value = [
             ({'id': 'id1'}, 'scope1'),
@@ -139,26 +155,29 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
             ({'id': 'id2'}, 'scope2'),
             ({'id': 'id3'}, 'scope3')]
         query.reset_mock()
-        result = list(
-            l3_db.L3_NAT_dbonly_mixin._unique_floatingip_iterator(query))
-        query.order_by.assert_called_once_with(l3_db.FloatingIP.id)
-        self.assertEqual([({'id': 'id1'}, 'scope1'),
-                          ({'id': 'id2'}, 'scope2'),
-                          ({'id': 'id3'}, 'scope3')], result)
+        with mock.patch.object(
+                l3_obj.FloatingIP, '_load_object',
+                side_effect=({'id': 'id1'}, {'id': 'id2'}, {'id': 'id3'})):
+            result = list(
+                l3_obj.FloatingIP._unique_floatingip_iterator(context, query))
+            query.order_by.assert_called_once_with(l3_models.FloatingIP.id)
+            self.assertEqual([({'id': 'id1'}, 'scope1'),
+                              ({'id': 'id2'}, 'scope2'),
+                              ({'id': 'id3'}, 'scope3')], result)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_deletion_port_not_found(self, gp):
         # port not found doesn't prevent
         gp.return_value.get_port.side_effect = n_exc.PortNotFound(port_id='1')
         self.db.prevent_l3_port_deletion(None, None)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_device_owner_not_router(self, gp):
         # ignores other device owners
         gp.return_value.get_port.return_value = {'device_owner': 'cat'}
         self.db.prevent_l3_port_deletion(None, None)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_no_fixed_ips(self, gp):
         # without fixed IPs is allowed
         gp.return_value.get_port.return_value = {
@@ -167,7 +186,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
         }
         self.db.prevent_l3_port_deletion(None, None)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_no_router(self, gp):
         # without router is allowed
         gp.return_value.get_port.return_value = {
@@ -175,10 +194,10 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
             'device_id': '44', 'id': 'f',
             'fixed_ips': [{'ip_address': '1.1.1.1', 'subnet_id': '4'}]}
         self.db.get_router = mock.Mock()
-        self.db.get_router.side_effect = l3.RouterNotFound(router_id='44')
+        self.db.get_router.side_effect = l3_exc.RouterNotFound(router_id='44')
         self.db.prevent_l3_port_deletion(mock.Mock(), None)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_existing_router(self, gp):
         gp.return_value.get_port.return_value = {
             'device_owner': n_const.DEVICE_OWNER_ROUTER_INTF,
@@ -188,53 +207,60 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
         with testtools.ExpectedException(n_exc.ServicePortInUse):
             self.db.prevent_l3_port_deletion(mock.Mock(), None)
 
-    @mock.patch.object(manager.NeutronManager, 'get_plugin')
+    @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_existing_floating_ip(self, gp):
+        ctx = context.get_admin_context()
         gp.return_value.get_port.return_value = {
             'device_owner': n_const.DEVICE_OWNER_FLOATINGIP,
             'device_id': 'some_flip', 'id': 'f',
             'fixed_ips': [{'ip_address': '1.1.1.1', 'subnet_id': '4'}]}
-        self.db.get_floatingip = mock.Mock()
-        with testtools.ExpectedException(n_exc.ServicePortInUse):
-            self.db.prevent_l3_port_deletion(mock.Mock(), None)
+        with mock.patch.object(l3_obj.FloatingIP, 'objects_exist',
+                               return_value=mock.Mock()),\
+            testtools.ExpectedException(n_exc.ServicePortInUse):
 
-    @mock.patch.object(l3_db, '_notify_subnetpool_address_scope_update')
-    def test_subscribe_address_scope_of_subnetpool(self, notify):
-        l3_db.L3RpcNotifierMixin._subscribe_callbacks()
+            self.db.prevent_l3_port_deletion(ctx, None)
+
+    @mock.patch.object(directory, 'get_plugin')
+    def test_subscribe_address_scope_of_subnetpool(self, gp):
+        l3_db.L3RpcNotifierMixin()
         registry.notify(resources.SUBNETPOOL_ADDRESS_SCOPE,
-                        events.AFTER_UPDATE, mock.ANY, context=mock.ANY,
+                        events.AFTER_UPDATE, mock.ANY,
+                        context=mock.MagicMock(),
                         subnetpool_id='fake_id')
-        notify.assert_called_once_with(resources.SUBNETPOOL_ADDRESS_SCOPE,
-                                       events.AFTER_UPDATE, mock.ANY,
-                                       context=mock.ANY,
-                                       subnetpool_id='fake_id')
+        self.assertTrue(gp.return_value.notify_routers_updated.called)
 
     def test__check_and_get_fip_assoc_with_extra_association_no_change(self):
         fip = {'extra_key': 'value'}
         context = mock.MagicMock()
-        floatingip_db = l3_db.FloatingIP(
-            id='fake_fip_id',
-            floating_network_id='fake_floating_network_id',
-            floating_ip_address='8.8.8.8',
-            fixed_port_id='fake_fixed_port_id',
-            floating_port_id='fake_floating_port_id')
+        floatingip_obj = l3_obj.FloatingIP(
+            context,
+            id=uuidutils.generate_uuid(),
+            floating_network_id=uuidutils.generate_uuid(),
+            floating_ip_address=netaddr.IPAddress('8.8.8.8'),
+            fixed_port_id=uuidutils.generate_uuid(),
+            floating_port_id=uuidutils.generate_uuid())
         with mock.patch.object(
                 l3_db.L3_NAT_dbonly_mixin,
                 '_get_assoc_data',
                 return_value=('1', '2', '3')) as mock_get_assoc_data:
-            self.db._check_and_get_fip_assoc(context, fip, floatingip_db)
+            self.db._check_and_get_fip_assoc(context, fip, floatingip_obj)
             context.session.query.assert_not_called()
             mock_get_assoc_data.assert_called_once_with(
-                mock.ANY, fip, floatingip_db)
+                mock.ANY, fip, floatingip_obj)
 
     def test__notify_attaching_interface(self):
         with mock.patch.object(l3_db.registry, 'notify') as mock_notify:
             context = mock.MagicMock()
             router_id = 'router_id'
             net_id = 'net_id'
-            self.db._notify_attaching_interface(context, router_id, net_id)
+            router_db = mock.Mock()
+            router_db.id = router_id
+            port = {'network_id': net_id}
+            intf = {}
+            self.db._notify_attaching_interface(context, router_db, port, intf)
             kwargs = {'context': context, 'router_id': router_id,
-                      'network_id': net_id}
+                      'network_id': net_id, 'interface_info': intf,
+                      'router_db': router_db, 'port': port}
             mock_notify.assert_called_once_with(
                 resources.ROUTER_INTERFACE, events.BEFORE_CREATE, self.db,
                 **kwargs)
@@ -246,7 +272,7 @@ class L3_NAT_db_mixin(base.BaseTestCase):
         self.db = l3_db.L3_NAT_db_mixin()
 
     def _test_create_router(self, external_gateway_info=None):
-        router_db = l3_db.Router(id='123')
+        router_db = l3_models.Router(id='123')
         router_dict = {'id': '123', 'tenant_id': '456',
                        'external_gateway_info': external_gateway_info}
         # Need to use a copy here as the create_router method pops the gateway
@@ -282,3 +308,11 @@ class L3_NAT_db_mixin(base.BaseTestCase):
                                 {'subnet_id': 'subnet-id',
                                  'ip_address': 'ip'}]}
         self._test_create_router(ext_gateway_info)
+
+    def test_add_router_interface_no_interface_info(self):
+        router_db = l3_models.Router(id='123')
+        with mock.patch.object(l3_db.L3_NAT_dbonly_mixin, '_get_router',
+                               return_value=router_db):
+            self.assertRaises(
+                n_exc.BadRequest,
+                self.db.add_router_interface, mock.Mock(), router_db.id)
